/*===========================================================================
	  Copyright 1998 (c) Electronic Data Systems Corporation
	                  Moscow Office 
	        Unpublished - All rights reserved
                   PROGRAM NAME:  m100.c
                   AUTHOR:        Chizh Oleg
                   CREATION DATE: 06-06-1998 
		   VERSION:       2.0
===========================================================================*
/***************************************************************************
 *                                                                         *
 * This is a C program written for Internal User Function.                 *
 * Its is postprocessor mit non standart cycles for NC machine M100        *
 * Voest-Alpine and controller Sinumeric 840C                              *
 *                                                                         *
 *                                                                         *
 ***************************************************************************
/*  This source code compile, link and testing for HP-UX v10.20            *
 *                         and for Windows NT v4.0 with Service Pack 3     *
 ***************************************************************************/
#define M_PI 3.14159265358979 

/* Include files */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <fcntl.h>
#include <errno.h>
#include <math.h>
#include <ctype.h>
#include <wchar.h>

#include <uf.h>
#include <uf_cfi.h>
#include <uf_ui.h>
#include <uf_object_types.h>
#include <uf_disp.h>
#include <uf_modl.h>
#include <uf_curve.h>
#include <uf_defs.h>
#include <uf_exit.h>
#include <uf_csys.h>
#include <uf_part.h>

/* Include file for declaration structures tool parameters */

/* 
  EXAMPLE STRING FOR STD_MILL_TOOL 5 PARAMETERS
  TLDATA/MILL,30.0000,4.0000,75.0000,0.0000,0.0000,11.0000,4.0000

  EXAMPLE STRING FOR STD_MILL_TOOL 7 PARAMETERS
  TLDATA/MILL,30.0000,4.0000,75.0000,0.0000,0.0000,11.0000,4.0000

  EXAMPLE STRING FOR STD_MILL_TOOL 10 PARAMETERS
  TLDATA/MILL,30.0000,40.0000,75.0000,0.0000,0.0000,0.0000,40.0000,4.0000,11.0000,4.0000
*/

typedef struct STD_MILL_TOOL
{
	double diameter,    low_radius,  length,      taper_angle;
	double tip_angle,   x_center_r1, y_center_r1, upper_radius;
	double x_center_r2, y_center_r2;
} SDT_MILL_TOOL_t, *STD_MILL_TOOL_p_t;

/* 
  EXAMPLE STRING FOR BARREL_MILL_TOOL
  TLDATA/BARREL,25.0000,38.0000,1.0000,2.0000,30.0000,25.0000,12.5000,9.5000
*/

typedef struct BARREL_MILL_TOOL
{
	double diameter, bar_radius,   low_radius,  upp_radius;
	double length,   flute_length, y_center_r1, shank_diameter;
} BARREL_MILL_TOOL_t, *BARREL_MILL_TOOL_p_t;

/*
  EXAMPLE STRING FOR TCUTTER_MILL_TOOL
  TLDATA/TCUTTER,25.0000,1.0000,2.0000,50.0000,13.0000,9.5000
*/

typedef struct TCUTTER_MILL_TOOL
{
	double diameter, low_radius, upp_radius, length, flute_length, shank_diameter;
} TCUTTER_MILL_TOOL_t, *TCUTTER_MILL_TOOL_p_t;

/* 
  EXAMPLE STRING FOR DRILL_TOOL (TURN MODE)
  TLDATA/DRILL,LATHE,8.0000,0.0000,50.0000,118.0000,35.0000 
  
  EXAMPLE STRING FOR DRILL_TOOL (MILL MODE)
  TLDATA/DRILL,MILL,8.0000,0.0000,50.0000,118.0000,35.0000 
*/

typedef struct DRILL_TOOL
{
 char module_type[6];
	double diameter, corner_radius, length, point_angle, flute_length;
} DRILL_TOOL_t, *DRILL_TOOL_p_t;

/* 
  EXAMPLE STRING FOR STD_TURN_TOOL
  TLDATA/TURN,LEFT,OUTSIDE,0.0000,30.0000,5.0000,45.0000,150.0000,270.0000
*/

typedef struct STD_TURN_TOOL
{
	char tool_orient[6], type_of_work[8];
	double noze_diam, holder_width, lead_angle, heel_angle, holder_heigth, orient_angle;
} STD_TURN_TOOL_t, *STD_TURN_TOOL_p_t;

/* 
  EXAMPLE STRING FOR BUTTON_TURN_TOOL
  TLDATA/TURN,BUTTON,LEFT,OUTSIDE,12.7000,9.5000,150.0000,270.0000
*/

typedef struct BUTTON_TURN_TOOL
{
	char tool_orient[6], type_of_work[8];
	double noze_diam, holder_width, holder_heigth, orient_angle;
} BUTTON_TURN_TOOL_t, *BUTTON_TURN_TOOL_p_t;

/* EXAMPLE STRING FOR BAR_TURN_TOOL
TLDATA/TURN,BORBAR,RIGHT,OUTSIDE,4.0000,50.0000,
5.0000,5.0000,125.0000,25.0000,180.0000 */

typedef struct BAR_TURN_TOOL
{
	char tool_orient[6], type_of_work[8];
	double noze_diam, holder_width, lead_angle, heel_angle, bar_radius, holder_heigth, orient_angle;
} BAR_TURN_TOOL_t, *BAR_TURN_TOOL_p_t;

/*
  EXAMPLE STRING FOR STD_GROOVE_TURN_TOOL
  TLDATA/GROOVE,RING,LEFT,OUTSIDE,TL,6.2500,8.0000,30.0000,45.0000,45.0000,0.0000,40.0000,70.0000
*/

typedef struct STD_GROOVE_TURN_TOOL
{
	char tool_orient[6], type_of_work[8], trace_size[3];
	double insert_width, left_radius, rigth_radius, max_depth, lead_angle;
	double insert_angle, holder_height, holder_width;
} SDT_GROOVE_TURN_TOOL_t, *STD_GROOVE_TURN_TOOL_p_t;

/*
  EXAMPLE STRING FOR ANGLE_GROOVE_TURN_TOOL
  TLDATA/GROOVE,RING,LEFT,OUTSIDE,TL,6.2500,8.0000,30.0000,45.0000,45.0000,0.0000,40.0000,70.0000 */

typedef struct ANGLE_GROOVE_TURN_TOOL
{
	char tool_orient[6], type_of_work[8], trace_size[3];
	double insert_width, left_angle, rigth_angle, max_depth, lead_angle;
	double insert_angle, holder_height, holder_width;
} ANGLE_GROOVE_TURN_TOOL_t, *ANGLE_GROOVE_TURN_TOOL_p_t;

/* 
  EXAMPLE STRING FOR STD_THREAD_TURN_TOOL
  TLDATA/THREAD,LEFT,OUTSIDE,20.0000,30.0000,60.0000,75.0000,150.0000,6.2500,3.5000 */

typedef struct STD_THREAD_TURN_TOOL
{
	char tool_orient[6], type_of_work[8];
	double max_depth, holder_width, refief_angle, heel_angle;
	double holder_height, insert_width, tip_offset;
} SDT_THREAD_TURN_TOOL_t, *STD_THREAD_TURN_TOOL_p_t;

/* 
  EXAMPLE STRING FOR BUT_THREAD_TURN_TOOL
  TLDATA/THREAD,BUTTRESS,LEFT,OUTSIDE,20.0000,30.0000,5.0000,40.0000,2.0000,150.0000,6.2500,1.2500 */

typedef struct BUT_THREAD_TURN_TOOL
{
	char tool_orient[6], type_of_work[8];
	double max_depth, holder_width, refief_angle, heel_angle, nose_radius;
	double holder_height, insert_width, tip_offset;
} BUT_THREAD_TURN_TOOL_t, *BUT_THREAD_TURN_TOOL_p_t;

/* 
  EXAMPLE STRING FOR ACME_THREAD_TURN_TOOL
  TLDATA/THREAD,ACME,LEFT,OUTSIDE,20.0000,30.0000,60.0000,60.0000,0.0000,150.0000,6.3500,3.5000 */

typedef struct ACME_THREAD_TURN_TOOL
{
	char tool_orient[6], type_of_work[8];
	double max_depth, holder_width, refief_angle, heel_angle;
	double nose_width, holder_height, insert_width, tip_offset;
} ACME_THREAD_TURN_TOOL_t, *ACME_THREAD_TURN_TOOL_p_t;


/* Functions declaration */

int  f_tldata(char *str_1);
void f_coolnt(char *str_1);
void f_delay (char *str_1);
void f_spindl(int chanel_in, int mill_mode, char *str_1);
void cycle_process      (int chanel_in,double b,
                         double *x,double *y, double *z,  
                         double *i,double *j, double *k );
void f_gohome           (int mill_mode, char *str_1);

int  f_load_tool(double *coord_b1, int multi_axis, char *str_1);
int  f_cutcom(char *str_1);

/* Function in service_fun.c */

struct msys { double p[9]; double c; };

void delete_right_zero  ( char *str_1);
void PTP                ( char *s);
void PTP_STOP           (char *message);
void PTP_CLOSE          (char *message);
void reset_XYZC         ();
void circle_process_mill( double B,
                          double coord_x,  double coord_y,   double coord_z, 
                          double Center_x, double Center_y,  double Center_z, 
                          double Normal_x, double Normal_y,  double Normal_z, 
                          double End_x,    double End_y,     double End_z,
                          double radius,   double tolerance, double ratio, double angle );
void position_out       ( double *position);
void B_rotate           ( double *x, double *z, double xoff, double	zoff, double b );
void C_rotate           ( double *y, double *z, double c);	
void c_calculation      ( double i,  double  j, double k, double *cr, double *c);
void   circle_unite     ( double  Xc, double  Yc, double  Zc,
                          double  Nx, double  Ny, double  Nz, double R,
                          double *Xe, double *Ye, double *Ze, double *angle);
void f_compensation	    (char *s, int *com, int *type_of_comp);                           


int  ask              ( char *si, char *kyeword, double *x);
int  XYZ_input        ( char *s, double *x, double *y, double *z, double *i, double *j, double *k, int mill_mode);
int  look_goto        ( double *b_coor);
int  circle_direction ( double Bx, double By, double Bz,
                        double Cx, double Cy, double Cz,
                        double Ex, double Ey, double Ez,
                        double Nx, double Ny, double Nz );
int  compare_msys     ( struct msys m1, struct msys m2);

char   *add_cls(char *out, char *inp);
char   *add_number(char *s);
char   *add_feedrate(char *s);

double around        ( double x);
double Len2          ( double x, double y);
double Len3          ( double x, double y, double z);
double b_calculation ( double i, double j, double k);
double c_correction  ( double c); 

/* Measuring function */

void f_l822(), f_l823(), f_l824(), f_l825(), f_l827(), f_l828(), f_l830(), f_l831(), f_l832();
void f_l833(), f_l834(), f_m56(),  f_m57 (), f_m13 (), f_m14 (), f_l9999();
void f_l930(int zero_shift), f_l931(int zero_shift), f_m19(int mill_mode);

/* Global variable */

char file_name[256] = "\0";
char cls_file[256], ptp_file[256], str_goto_f[256];
char nc_block_in[256], nc_block_out[256], str_goto[256];

/* TOOL DATA DEFINITION */

SDT_MILL_TOOL_t          mill_tool;
BARREL_MILL_TOOL_t       barrel_tool;
TCUTTER_MILL_TOOL_t      tcutter_tool;
DRILL_TOOL_t             drill_tool;
STD_TURN_TOOL_t          turn_tool;
BUTTON_TURN_TOOL_t       button_tool;
BAR_TURN_TOOL_t          bar_tool;
SDT_GROOVE_TURN_TOOL_t   groove_tool;
ANGLE_GROOVE_TURN_TOOL_t angle_groove_tool;
SDT_THREAD_TURN_TOOL_t   thread_tool;
BUT_THREAD_TURN_TOOL_t   but_thread_tool;
ACME_THREAD_TURN_TOOL_t  acme_thread_tool;

int    i_cancel=2;                    /* the code return after CANSEL command in dialog */
int    fedrat_on;	                    /* set of feed rate command */
int    type_of_goto=-1;             /* -1 unknown, 0-G0, 1-G1, 2-G2, 3-G3 */
char   feedrate[12];                       /* string with feedrate command */
int    plane_of_circle= -1;	/* 1 - Y1 x Z1 (	E1=const normal to tool axis)
				2 - E1 x Y1 ( normal of cricle || to acis of spindel )
				3 - Z1 x E1 ( common plane of circle in turn operation */
int    tool_sys;                           /* set of tool sys coordinat (command l843) */
int    block_num=1, chanel_out, chanel_in; 
int    i_rotate_c=0;                       /* flag of C rotation on 180 degree */
int    mill_mode=-1;                       /* flag of milling operation */
int    from_move = 0;                      /* this variable set a flag of FROM POINT in operation */		 
int    tool_change=1;                      /* this variable calculate a numbers of tool change cycles */
																																													
double Xold,Yold,Zold,Cold;		              /* the old coordiant from CLS line */
double rotate;		                           /* curren angle of C correction */
double tmp1,tmp2,tmp3;

// #define VERSION 19          /* Version of Unigraphics */
// #define SERVER_NAME "chizh" /* Define server name */
// #define WINNT   0           /* Type operation system 1-WNT 0-UNIX */

void ufusr(char *param, int *retcod, int param_len)
{

/* Variables in main program */

   char s3[30] = ""; 
   char inp_values[1][31]={1};
   char *mes_prog_num="Enter Program Number";
   char menu_list[1][16]={"Program number"};
   char menu_list_1[2][38]={"Not using 5 axis simultaneous", 
                            "Using 5 axis simultaneous"};
   double initial_d = 0.0; 
   int type_item[1]={100};
   char tool_path[256];

/* Variable of menu tool change cycle L106 */
   int     r_param_i[2] = {1, 0};
   int     tool_items[2] = {100, 100};
   double  r_param_d[2] = {0.0, 0.0};
   char    tool_menu_list[2][16] = {"R1=", "R3="};   
   char    r_param_c[2][31] = {"0", "0"};
   char    *message_tool="Input parameters in tool change cycle for tool N ";
   char    message_tool_1[128];
/*--------------------------------------------*/
   char   *s1, *filter = "*.cls", error[133];
// char *s2;
   int    type_operation, t_orient = 0, rapid_mod, len_str;
   int    tool_num, part_no = 1; 
   int    comp, type_of_comp=0, irc, in ;
   int    rapid_move = 1, move_mode = 1, lin_move=1, angle_move, multi_axis = 1,first_move = 0;
   int    upcom=0, done_comp=0;
   double tool_angle, d_fedrat, coord_b1 = 0.0;
   double Iold, Jold, Kold;
   double origin_x, origin_y, origin_z;
   double feed_rate_comp;
   double tolerance, ratio, angle;
   double E1,E1_old,Z1,Z1_old,Y1,Y1_old,B1,B1_old,C1,C1_old;
   double Tool,dE1,dZ1,dY1,dC1,dB1,dX,dY,dZ;

/* THREAD PARAMETER */
   
   double d_pitch,d_varpinch;
   int    i_decr,i_incr,i_thread_param,i_turn;


/* Variables functions in CAM operation create */
  
   char *l930="L930", *l931="L931", *m56="M56", *m57="M57", *m13="M13", *m14="M14";
   char *l822="L822", *l823="L823", *l824="L824", *l825="L825", *l827="L827"; 
   char *l828="L828", *l830="L830", *l831="L831", *l832="L832", *l833="L833"; 
   char *l834="L834", *m19="M19", *l9999="L9999", *l843="L843", *l844="L844"; 

/* Variables function in CLS FILE */
  
   char *tldata = "TLDATA/", *coolnt = "COOLNT/", *delay = "DELAY/", *disply = "DISPLY/";
   char *fedrat_mmpr = "FEDRAT/MMPR", *fedrat_mmpm = "FEDRAT/MMPM";
   char *fedrat = "FEDRAT/";
   char *gohome = "GOHOME/", *load_tool = "LOAD/TOOL", *opstop = "OPSTOP";
   char *origin = "ORIGIN/", *partno = "PARTNO/", *rapid = "RAPID", *m_goto = "GOTO/";
   char *set_turn = "SET/MODE,TURN", *set_mill = "SET/MODE,MILL", *turret = "TURRET/";
   char *select = "SELECT/TOOL", *cutcom = "CUTCOM/", *spindl = "SPINDL/";
   char *cycle = "CYCLE/", *cycle_off = "CYCLE/OFF", *from = "FROM/", *circle = "CIRCLE/";
   char *thread_turn="THREAD/TURN", *thread_on="THREAD/ON", *thread_face="THREAD/FACE";
    
   char COMMAND[100];
   int  units = UF_PART_METRIC;
   int  response = UF_UI_OK;
   int  max_speed, zero_shift = 54;
   char *mes_sel_zero = "Select zero shift";
   char menu_list_23[3][38]={"MAX Speed = 1600 rev.", "MAX Speed = 560 rev.", "MAX Speed = 1000 rev."};
   char menu_list_10[4][38]={"G54", "G55", "G56", "G57"};
   char menu_list_11[3][38]={"Tool angle = 0 degree", "Tool angle = -90 degree", "Input tool angle"};
   char *mes_tool_angle = "Input tool orientation angle";
   char menu_list_12[1][16]={"Tool angle"};
   char *s_name="";
   /* --<new variable>-------*/

   int    CM,file_pos;									
   int	  meas_tool; /* if measurnig 1, else 0 */
   int    out_move;   /* control GOTO output by coordinat modality*/

   double X,Y,Z,I,J,K;    /* the current coordiant from CLS line */
   double Xout,Yout,Zout = 0.0,C,Cr,Br; /* the current coordiant from CLS line */

   double Xe,Ye,Ze, Xc,Yc,Zc, Nx,Ny,Nz, R;  /* end point, center, normal and R of circle */
   double Xoff,Zoff;                        /* the shift for B rotation */
   double Lpart,Lmas;
/* MSYS parameter */
   struct msys nm,  m[10];
   int    n=0, cur, i;  /* n - number of msys,   curr - currrent msys index */
   int    rel_num=0;
//   char   open_part[133], *relnum = "V19";
//   tag_t  part_tag;
   char   *str_1 = " ";
   char   *str_2 = "Version of UNIGRAPHICS must be ";

   int serv_name = 0;

/* DEBUGGING */

   double Yc_cls,Zc_cls,Y_goto,Z_goto,Cr_goto;
   Xold=Yold=Zold=Cold=coord_b1=-999999.99;
/* Reset of coordinate */

   UF_initialize();    /* Initialize UGOPEN program */
/*
   part_tag = UF_PART_ask_display_part();
   UF_PART_ask_part_name(part_tag, open_part);
   uc4623(open_part, relnum);
   str_1=(strstr(relnum, "V"));  
   sscanf(str_1+1, "%d", &rel_num); 
   sprintf(str_2, "%s%d", str_2, VERSION);

#if (WINNT)
   system("echo %ACS_MASTER% > d:\\temp\\acs_name");
   chanel_in = uc4504("d:\\temp\\acs_name", 1, 2004);
#else
   system("echo $ACS_MASTER > /tmp/acs_name");
   chanel_in = uc4504("/tmp/acs_name", 1, 2004);
#endif
   if(chanel_in < 0)
   {
      UF_get_fail_message(chanel_in, error);
      uc1601(error, 1);
   }
   while((in = uc4514(chanel_in, nc_block_in) >= 0))
   {
      strcpy(s_name, SERVER_NAME);
      s2 = strstr(nc_block_in, SERVER_NAME);
#if (WINNT)
      strncpy(s3, s2, strlen(s2)-1);
#else
      strncpy(s3, s2, strlen(s2));
#endif
      if((strcmp(s3, s_name) == 0)) serv_name = 1;
      else serv_name = 0;
   }
   uc4540(chanel_in, 0);

#if (WINNT)
   system("del  d:\\temp\\acs_name");
#else
   system("rm /tmp/acs_name");
#endif
      
   if(rel_num > VERSION || serv_name != 1) 
   {
      if(rel_num > VERSION)
      {
         uc1601(str_2, 1);
      }
      else
      {
         uc1601("Cannot connect to ACS Server", 1);
      }
   }
   else if((irc = uc5000(s3)) == 0)
   {
      uc1601("Not active part. Pleas create new part or open existing", 1);
   }
   else
   {
*/
      /* Set selection mask for CLS files */
      UF_UI_set_dialog_filter(UF_UI_PART_NEW_FLTR, filter);
      /* Select CLS file for Postprocessing */
      irc = UF_UI_ask_create_part_filename(file_name, &units, &response);
      if(irc)
      {
         UF_get_fail_message(irc,error); 
         uc1601(error, 1); 
         exit(1);
      }
      /* Convert string file name to ptp and dnc file and running GPM */
      len_str = strlen(file_name);
      strncat(ptp_file, file_name, (len_str-3));
      strcat(ptp_file, "doc");

      /*  Open files to input and output */

      chanel_in = uc4504(file_name, 1, 2004);
      if(chanel_in < 0)	{ UF_get_fail_message(chanel_in, error); uc1601(error, 1);	}

      chanel_out = uc4504(ptp_file, 3, 2004);
      if(chanel_out < 0) { UF_get_fail_message(chanel_out, error); uc1601(error, 1); }

      /* Input program number */

      irc = uc1613(mes_prog_num, menu_list, 1, &part_no, &initial_d, 
      inp_values, type_item);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
      sprintf(nc_block_out, "(%%MPF%d)", part_no);	uc4524(chanel_out, nc_block_out);
   
      /* Selection mode of cutting */

      mes_prog_num = "Select works mode (Fixed or Variable Axis)";
      irc = uc1603(mes_prog_num, 1, menu_list_1, 2); 
      if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
      if (irc==6) multi_axis = 1;
	  else        multi_axis = 0;
  
      /* Select zero shift and maximum speed of spindel  */

      mes_sel_zero = "Select zero shift";
      irc = uc1603(mes_sel_zero, 1, menu_list_10, 4);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
      switch(irc)
      {
         case  5: zero_shift = 54;	break;
         case  6: zero_shift = 55;	break;
         case  7: zero_shift = 56;	break;
         case  8: zero_shift = 57;	break;
         default: zero_shift = 54;
      }
      sprintf(nc_block_out, " G%d",zero_shift);	PTP(nc_block_out);

      mes_sel_zero="Select maximum speed of spindel rotation";
      irc = uc1603(mes_sel_zero, 1, menu_list_23, 3);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
      switch(irc)
      {
         case 5 : max_speed = 1600;	break;     
         case 6 : max_speed = 560 ;	break;
         case 7 : max_speed = 1000;	break;
         default: max_speed = 1600;
      }
      sprintf(nc_block_out, " G26 S1=%d", max_speed); 	PTP(nc_block_out);

      /* ==================> Read CLS file while not read EOF <===========================*/

      while((in = uc4514(chanel_in, nc_block_in) >= 0))
      { 
      /* TOOL PATH/ */		 
      if ((s1 =strstr(nc_block_in, "TOOL PATH/"))!=NULL)
      {
         strcat(strcat(strcpy(nc_block_out," ("),s1+10),")");	PTP(nc_block_out);
         strcpy(tool_path,nc_block_in);
      }
      else if ((s1=strstr(nc_block_in,"PAINT/"))!=NULL) { ; }
      /* MSYS */  
      else if ((s1=strstr(nc_block_in,"MSYS/"))!=NULL)
      {
         sscanf(s1+5,"%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf",
	     	&nm.p[0],&nm.p[1],&nm.p[2],&nm.p[3],&nm.p[4],&nm.p[5],&nm.p[6],&nm.p[7],&nm.p[8]);
		if (n==0){ cur=0; m[cur]=nm; m[cur].c=rotate=0.0; }
		else
		{
            for (i=0, cur=-1; i<=n; i++) if (compare_msys(m[i],nm)) cur=i;
            if  (cur==-1) { cur= (++n); m[cur]=nm; m[cur].c=rotate=-999999.999; }
         }  
      /* sprintf (nc_block_out," ( MSYS N=%d, C_ROTATE=%lf )", cur, m[cur].c); PTP(nc_block_out); */
      }
      /* ROTATE */ 
      else if ((s1=strstr(nc_block_in,"ROTATE/CAXIS"))!=NULL)
      {
		sscanf(s1+13,"%lf",&rotate); rotate = around(rotate); m[cur].c = rotate;
		sprintf (nc_block_out," G68 G0 C1=%.3lf",rotate); 
		PTP(nc_block_out);
		sprintf (nc_block_out," ( after ROTATE: MSYS N=%d, C_ROTATE=%lf )", cur, m[cur].c); 
		PTP(nc_block_out);
      }
      /* TLDATA/ */   
      else if((s1 = strstr(nc_block_in, tldata)) != NULL) 
      {
         type_operation = f_tldata(nc_block_in);
         if(type_operation == 5)
         {
            if((s1 = strstr(turn_tool.tool_orient, "RIG")) != NULL) t_orient = 180;
            else t_orient = 0;
            tool_angle = turn_tool.orient_angle - 270.0;
         }
         else if(type_operation == 6)
         {
            if((s1 = strstr(button_tool.tool_orient, "RIG")) != NULL) t_orient = 180;
            else t_orient = 0;
            tool_angle = button_tool.orient_angle - 270.0;
         }
         else if(type_operation == 7)
         {
            if((s1 = strstr(bar_tool.tool_orient, "RIG")) != NULL) t_orient = 180;
            else t_orient = 0;
            tool_angle = bar_tool.orient_angle - 270.0;
         }
         else if(type_operation == 8)
         {
			if((s1 = strstr(groove_tool.tool_orient, "RIG")) != NULL) t_orient = 180;
            else t_orient = 0;
            irc = uc1603(mes_sel_zero, 1, menu_list_11, 3);	
			if(irc==i_cancel) PTP_CLOSE(" PTP close by CANCEL comand");
            switch(irc)
            {
               case  5: tool_angle = 0.0;    break;
               case  6: tool_angle = -90.0;  break;
               case  7: uc1609(mes_tool_angle, menu_list_12, 1, &tool_angle, &irc);   break;
               default: tool_angle = 0.0;
            }
         }
         else if(type_operation == 9)
         {
            if((s1 = strstr(angle_groove_tool.tool_orient, "RIG")) != NULL) t_orient = 180;
            else t_orient = 0;
            irc = uc1603(mes_sel_zero, 1, menu_list_11, 3);	
			if(irc==i_cancel) PTP_CLOSE(" PTP close by CANCEL comand");
            switch(irc)
            {
               case  5: tool_angle = 0.0;    break;
               case  6: tool_angle = -90.0;  break;
               case  7: uc1609(mes_tool_angle, menu_list_12, 1, &tool_angle, &irc);   break;
               default: tool_angle = 0.0;
            }
         } 
         else if(type_operation == 11)
         {
            t_orient = 0;
            tool_angle = -90.0; 
         }
         else 
         {
            t_orient = 0;
            tool_angle = 0.0; 
         } 
      }
      /* SET/TURN */   
      else if((s1 = strstr(nc_block_in, set_turn)) != NULL) 
      {
         PTP(" G90");	PTP(" L920");  mill_mode = 0; Xold=Yold=Zold=-999999.99;
      }
      /* SET/MILL */   
      else if((s1 = strstr(nc_block_in, set_mill)) != NULL) 
      {
         PTP(" G90"); PTP(" L918"); mill_mode = 1;		Xold=Yold=Zold=-999999.99;
      }
      /* COOLNT */   
      else if((s1 = strstr(nc_block_in, coolnt)) != NULL)	f_coolnt(s1);
      /* DELAY  */   
      else if((s1 = strstr(nc_block_in, delay)) != NULL)	f_delay (s1);
      /* DISPLAY */  
      else if((s1 = strstr(nc_block_in, disply)) != NULL)
      {
         strcat(strcat(strcpy(nc_block_out," ("),s1+7),")");	PTP(nc_block_out);
      }
      /* FEDRAT/MMPM or FEDRAT/MMPR */
      else if( (s1 = strstr(nc_block_in, fedrat_mmpr)) != NULL ||
               (s1 = strstr(nc_block_in, fedrat_mmpm)) != NULL)
      {
         sscanf(s1+12, "%lf", &d_fedrat); 
         d_fedrat=around(d_fedrat); sprintf (feedrate," F%.3lf",d_fedrat); 
		 delete_right_zero(feedrate);
         fedrat_on = 1;
      }
      /* FEDRAT */   
      else if((s1 = strstr(nc_block_in, fedrat)) != NULL)
      {
         sscanf(s1+7, "%lf", &d_fedrat);
         d_fedrat=around(d_fedrat); 
	 sprintf (feedrate," F%.3lf",d_fedrat); 
	 delete_right_zero(feedrate);
         fedrat_on = 1;
      }
      /* FROM */   
      else if((s1 = strstr(nc_block_in, from)) != NULL)
      {
         if((XYZ_input(s1+5, &X, &Y, &Z, &Iold,&Jold, &Kold, mill_mode)) == 3) {  Iold=Jold=0.0; Kold=1.0;}
	      if(mill_mode)	coord_b1=b_calculation(Iold,Jold,Kold);
         from_move = 1; type_of_goto=-1;	
         tool_sys=0; /* reset of tool coordinat system */
         Xold=Yold=Zold=-99999999.99;
      }
      /* LOAD/TOOL */   
		else if((s1 = strstr(nc_block_in, load_tool)) != NULL)	
		{
			if(!from_move &&  !look_goto(&coord_b1))
			{  
				PTP_STOP(" ERROR: not FROM and GOTO command with I,J,K parameters");
				break;
			}
			meas_tool=f_load_tool( &coord_b1, multi_axis, s1);
			if(tool_change > 200) tool_change = 1;
			tool_change++;
		}
       /* TURRET */      
       else if((s1 = strstr(nc_block_in, turret)) != NULL)
      {
         sscanf(s1+12, "%d", &tool_num);
         sprintf(message_tool_1, "%s%d", message_tool, tool_num);
         uc1613(message_tool_1, tool_menu_list, 2, r_param_i, r_param_d,
         r_param_c, tool_items);      
         strcpy(message_tool_1, "\0");
         sprintf(nc_block_out, " T1=%d R1=%d R3=%d R4=%.3lf R6=%d L106",
         tool_num, r_param_i[0], r_param_i[1], tool_angle, t_orient);
         PTP(nc_block_out);	
      }
      /* SELECT/TOOL */   
      else if((s1 = strstr(nc_block_in, select)) != NULL)
      {
         sscanf(s1+12, "%d", &tool_num);
         sprintf(nc_block_out, " T1=%d", tool_num++);
         PTP(nc_block_out);
      }
      /* UPCOM   */   
      else if((s1 = strstr(nc_block_in, "UPCOM"))!= NULL)	upcom=1;
      /* OPSTOPT */   
      else if((s1 = strstr(nc_block_in, opstop)) != NULL)	PTP(" M1");
      /* STOP    */   
      else if((s1 = strstr(nc_block_in, "STOP")) != NULL)	PTP(" M0");
      /* ORIGIN  */   
      else if((s1 = strstr(nc_block_in, origin)) != NULL)
		      sscanf(s1+7, "%lf,%lf,%lf", &origin_x, &origin_y, &origin_z);
      /* PART NO */   
      else if((s1 = strstr(nc_block_in, partno)) != NULL)	sscanf(s1+7, "%d", part_no);
      /* SPINDL  */   
      else if((s1 = strstr(nc_block_in, spindl)) != NULL)	f_spindl(chanel_in, mill_mode, nc_block_in);
      /* L930    */   
      else if((s1 = strstr(nc_block_in, l930))   != NULL)	f_l930(zero_shift);
      /* L931    */   
      else if((s1 = strstr(nc_block_in, l931))   != NULL)	f_l931(zero_shift);
      /* CYCLE   */   
		else if((s1 = strstr(nc_block_in, cycle))  != NULL)	
			cycle_process(chanel_in, coord_b1,&Xold,&Yold,&Zold,&Iold, &Jold, &Kold);
      /* CUTCOM  */   
      else if((s1 = strstr(nc_block_in, cutcom)) != NULL)
      {
         if(done_comp!=uc4534(chanel_in) ) 
			 f_compensation(nc_block_in, &comp, &type_of_comp);
      }
      /* GOHOME  */   
      else if((s1 = strstr(nc_block_in, gohome)) != NULL)	
		  f_gohome(mill_mode, nc_block_in);
      /* RAPID   */   
      else if((s1 = strstr(nc_block_in, rapid)) != NULL) 
      {
         rapid_mod = 1;
         move_mode = 1;
         strcpy(COMMAND,"G0");
      }
/* GOTO 5 Axis */
		else if((s1 = strstr(nc_block_in, m_goto)) != NULL && multi_axis == 1)
		{
/* the control of G0/G1 modality */
			if(rapid_mod && type_of_goto !=0 )     {type_of_goto=0; strcpy(COMMAND," G0");}
			else if(!rapid_mod && type_of_goto!=1) {type_of_goto=1; strcpy(COMMAND," G1");}
			else strcpy(COMMAND,"");           
			if((XYZ_input(s1+5,&X,&Y,&Z, &I,&J,&K, mill_mode)) == 3)
			{
				I = Iold; J = Jold; K = Kold;
			}
			if ((coord_b1 = b_calculation(I,J,K)) > 90.01 ||
				(coord_b1 = b_calculation(I,J,K)) < -90.01 ) 
			{ 
				printf("B1=%.3lf\n",coord_b1);
				PTP_STOP(" ERROR: B-axis out of range ");
				break;
			}         
			coord_b1 = b_calculation(I,J,K); c_calculation (I,J,K, &Cr,&C);
			C_rotate(&Y,&Z,Cr);	
/* calculation of real coordinates of NC mashine with tool lenght acount */
			Br = coord_b1/180.0*M_PI;
			E1=	Z+Tool*cos(Br); Y1=Y; Z1=X-Tool*sin(Br); C1=C; B1=coord_b1;
			E1=around(E1); Y1=around(Y1); Z1=around(Z1); C1=around(C1); B1=around(B1);
			if (type_of_goto == 1 && fedrat_on)
			{ 
				dE1=E1-E1_old; dZ1=Z1-Z1_old; dY1=Y1-Y1_old; dB1=B1-B1_old;
				dC1=fabs(C1-C1_old); 
				if(dC1 > 180.0) dC1 = 360.0-dC1; 
				dC1 = dC1/180.0*M_PI*fabs(E1+E1_old)/2.0;
				dX=X-Xold; dY=Y-Yold; dZ=Z-Zold;
				Lmas =sqrt(dE1*dE1+dZ1*dZ1+dY1*dY1+dC1*dC1+dB1*dB1);
				Lpart=sqrt(dX*dX+dY*dY+dZ*dZ);
				feed_rate_comp=Lmas/Lpart*d_fedrat;
				if(feed_rate_comp==0.0) feed_rate_comp=d_fedrat;
			}
			else if(fedrat_on) feed_rate_comp=d_fedrat; 
			else	feed_rate_comp=0.0;
			Br=coord_b1/180.0*M_PI;
			if(sin(Br) < 0.0) sprintf(nc_block_out," R701= R51*%.6lf ",-sin(Br));
			else	sprintf(nc_block_out," R701=-R51*%.6lf ",	sin(Br));
			sprintf(nc_block_out,"%sR700=R51*%.6lf",nc_block_out,cos(Br));
			PTP(nc_block_out);
			if(type_of_goto == 1) sprintf (nc_block_out,
					"%s G68 E1=%.3lf+R700 Y1=%.3lf Z1=%.3lf+R701 C1=%.3lf B1=%.3lf F%.3lf",
					COMMAND,Z,Y,X,C1,B1,feed_rate_comp);
			else sprintf(nc_block_out,
					"%s G68 E1=%.3lf+R700 Y1=%.3lf Z1=%.3lf+R701 C1=%.3lf B1=%.3lf",
					COMMAND,Z,Y,X,C1,B1);
			PTP(nc_block_out);
/* save what we need */
			Xold=around(X); Yold=Y; Zold=Z; Iold=I; Jold=J; Kold=K;
			E1_old=E1; Z1_old=Z1; Y1_old=Y1; C1_old=C1; B1_old=B1;	rapid_mod=0;
		}
/* GOTO 3Axis */
		else if((s1=strstr(nc_block_in,m_goto))!=NULL && (!multi_axis))  
		{
/* this part of code try to find CUTCOM command after GOTO 
and do compenastion output before GOTO command output it 
save the file position with founded and executed CATCOM command */
			if(upcom)
			{
				file_pos=uc4534(chanel_in);
				do
				{
					uc4514 (chanel_in,str_goto);
					if(strstr(str_goto,"GOTO/") != NULL || 
						strstr(str_goto,"RAPID/") != NULL || 
						strstr(str_goto ,"END-OF-PATH") !=NULL)
					{
						done_comp=0; break;
					}
					if(strstr(str_goto,"CUTCOM") != NULL && 
						strstr(str_goto,"OFF") == NULL)
					{
						done_comp = uc4534(chanel_in);
						f_compensation(str_goto, &comp, &type_of_comp); break;
					}		
				}while(1);	
				uc4535(chanel_in,file_pos);
			}
/* check was the FROM command in operation */
			if(!from_move && mill_mode && !meas_tool) 
			{
				PTP_STOP("ERROR: CSL operation MUST include FROM point");
				break;
			}
/* the control of G0/G1 modality */
			if( rapid_mod && type_of_goto!=0) { type_of_goto=0;       strcpy(COMMAND," G0"); }
			else if(!rapid_mod && type_of_goto!=1) { type_of_goto=1; strcpy(COMMAND," G1"); }
			else strcpy(COMMAND,""  );           
			angle_move = XYZ_input(s1+5,&X,&Y,&Z,&I,&J,&K,mill_mode); 
/* command without I,J,K save or GOTO or FROM */
			if(angle_move <= 3) { I=Iold, J=Jold,K= Kold;}	
			if(mill_mode == 1 && !tool_sys)  /* it is a normal milling operation */
			{	
				c_calculation(I,J,K,&Cr,&C);
				C_rotate(&Y,&Z,Cr);
				Xout=around(X); Yout=around(Y); Zout=around(Z); 
				C=c_correction(C); C=around(C); 
/* check the coordinate modality */
				CM=0;
				if(C != Cold)	 CM=CM+1;
				if(Xout != Xold) CM=CM+10;
				if(Yout != Yold) CM=CM+100;
				if(Zout != Zold) CM=CM+1000;
				out_move=1;
				switch (CM)
				{
					case 1111 : sprintf(nc_block_out," G68 E1=%.3lf Y1=%.3lf Z1=%.3lf C1=%.3lf",Zout,Yout,Xout,C); break;
					case 1110 : sprintf(nc_block_out,   " E1=%.3lf Y1=%.3lf Z1=%.3lf"          ,Zout,Yout,Xout  ); break;
					case 1101 : sprintf(nc_block_out," G68 E1=%.3lf Y1=%.3lf C1=%.3lf"         ,Zout,Yout,      C); break;
					case 1100 : sprintf(nc_block_out,   " E1=%.3lf Y1=%.3lf"                   ,Zout,Yout        ); break;
					case 1011 : sprintf(nc_block_out," G68 E1=%.3lf Z1=%.3lf C1=%.3lf"         ,Zout,      Xout,C); break;
					case 1010 : sprintf(nc_block_out,   " E1=%.3lf Z1=%.3lf"                   ,Zout,      Xout  ); break;
					case 1001 : sprintf(nc_block_out," G68 E1=%.3lf C1=%.3lf"                  ,Zout,            C); break;
					case 1000 : sprintf(nc_block_out,   " E1=%.3lf"                            ,Zout              ); break;
					case  111 : sprintf(nc_block_out," G68 Y1=%.3lf Z1=%.3lf C1=%.3lf"         ,      Yout,Xout,C); break;
					case  110 : sprintf(nc_block_out,   " Y1=%.3lf Z1=%.3lf"                   ,      Yout,Xout  ); break;
					case  101 : sprintf(nc_block_out," G68 Y1=%.3lf C1=%.3lf"                  ,      Yout,      C); break;
					case  100 : sprintf(nc_block_out,   " Y1=%.3lf"                            ,      Yout        ); break;
					case   11 : sprintf(nc_block_out," G68 Z1=%.3lf C1=%.3lf"                  ,            Xout,C); break;
					case   10 : sprintf(nc_block_out,   " Z1=%.3lf"                            ,            Xout  ); break;
					case    1 : sprintf(nc_block_out," G68 C1=%.3lf"                           ,                  C); break;
					default   : out_move=0;
				}
/* output of PTP command */ 
				if(out_move)			
				{
					sprintf(str_goto,"%s%s",COMMAND, nc_block_out);
					PTP(add_cls(add_feedrate(str_goto),nc_block_in));
				}
/* save what you need */
			Cold=C; Xold=Xout; Yold=Yout; Zold=Zout; 
			Iold=I; Jold=J; Kold=K;
			rapid_mod=0;
		}  
/* it is a L843 mode of milling operation */
		else if (mill_mode == 1 && tool_sys)  
		{	
			c_calculation(I,J,K,&Cr,&C);
			Y_goto=Y; Z_goto=Z; Cr_goto=Cr;
			C_rotate(&Y,&Z,Cr); B_rotate(&X,&Z,Xoff,Zoff,Br);
			Xout=around(X); Yout=around(Y); Zout=around(Z); 
			C=c_correction(C); C=around(C); 
/* check the coordinate modality */
			CM=0;
			if(Xout != Xold) CM = CM + 10;
			if(Yout != Yold) CM = CM + 100;
			if(Zout != Zold) CM = CM + 1000;
			out_move=1;
			switch (CM)
			{
				case 1110 : sprintf(nc_block_out, " E%.3lf Y%.3lf Z%.3lf" 
                             ,Xout, Yout, Zout); 
					break;
				case 1100 : sprintf(nc_block_out, " Y%.3lf Z%.3lf"        
                             ,Yout, Zout); 
					break;
				case 1010 : sprintf(nc_block_out, " E%.3lf Z%.3lf"        
                             ,Xout, Zout); 
				    break;
				case 1000 : sprintf(nc_block_out,   " Z%.3lf"   
                             ,Zout); 
					break;
				case  110 : sprintf(nc_block_out,   " E%.3lf Y%.3lf"
                             ,Xout, Yout); 
					break;
				case  100 : sprintf(nc_block_out,   " Y%.3lf"
                             ,Yout); 
					break;
				case   10 : sprintf(nc_block_out,   " E%.3lf" 
                             ,Xout); 
					break;
				default   : out_move=0;
			}
/* output of PTP command */
			if(out_move) 
			{
				sprintf(str_goto,"%s%s",COMMAND, nc_block_out);
				PTP(add_cls(add_feedrate(str_goto),nc_block_in));
			}
/* save what you need */
			Xold=Xout; Yold=Yout; Zold=Zout; 
			Iold=I; Jold=J; Kold=K;
			rapid_mod=0;
		}  
/* it is a Turning operation */
		else 
		{
			Xout = X; Yout = Y*2.0; Xout = around(Xout); Yout = around(Yout);
/* check the coordinate modality */
			CM=0;
			if(Xout != Xold) CM = CM + 1;
			if(Yout != Yold) CM = CM + 10;
			out_move=1;
			switch(CM)
			{
				case 11 : sprintf(nc_block_out, " X1=%.3lf Z1=%.3lf",
                           Yout, Xout); 
					break;
				case 10 : sprintf(nc_block_out, " X1=%.3lf",        
                           Yout); 
					break;
				case  1 : sprintf(nc_block_out, " Z1=%.3lf",  
                           Xout); 
					break;
				default : out_move=0;
			}
/* output of PTP command */
			if (out_move)
			{
				sprintf(str_goto,"%s%s",COMMAND, nc_block_out);
				PTP(add_cls(add_feedrate(str_goto),nc_block_in));
			}
/* save what you need */
			Xold=Xout; Yold=Yout; 
			rapid_mod=0;
		}  
/* end of turning block      */
	}
/* endof GOTO */
/* THEARED*/
	else if((s1=strstr(nc_block_in,"THREAD"))!=NULL	&& (!mill_mode)) 
	{
		if(strstr(nc_block_in,"THREAD/ON")==NULL)
		{
			i_thread_param = ask(nc_block_in, "PITCH,", &d_pitch);
			i_decr = ask(nc_block_in, "DECR,", &d_varpinch); 
			i_incr = ask(nc_block_in, "INCR,", &d_varpinch); 
			if(strstr(nc_block_in, "THREAD/TURN") != NULL)  i_turn = 1;
			if(strstr(nc_block_in, "THREAD/FACE") != NULL)  i_turn = 0; 
		}
		else if(!i_thread_param) { PTP(" ERROR thread with unknow parameters"); }
		if (i_decr) strcpy(COMMAND," G34");
		else if(i_incr) strcpy(COMMAND," G35");
		else            strcpy(COMMAND," G33");
		do	/* skip everything until first GOTO */
		{ 
			uc4514(chanel_in,nc_block_in); 
		} while(!ask(nc_block_in,"GOTO/", &Xout));
		XYZ_input(nc_block_in+5,&Xout,&Yout,&Zout,&tmp1,&tmp2,&tmp3,mill_mode);
		Xout=around(Xout); Yout=around(Yout);
		CM=0;
		if(Xout != Xold) CM = CM + 1;
		if(Yout != Yold) CM = CM + 10;
		out_move = 1;
		switch(CM)
		{
			case 11 : 		    sprintf(nc_block_out, " X1=%.3lf Z1=%.3lf",Yout,Xout); break;
			case 10 : 		    sprintf(nc_block_out, " X1=%.3lf",         Yout     ); break;
			case  1 : 		    sprintf(nc_block_out, " Z1=%.3lf",              Xout); break;
			default :       out_move=0;
		}
		if(out_move)	/* output of PTP command */
		{
			if(i_turn) sprintf(str_goto,"%s%s K%.3lf",COMMAND, nc_block_out,d_pitch);
			else       sprintf(str_goto,"%s%s I%.3lf",COMMAND, nc_block_out,d_pitch);
			if(i_decr || i_incr) sprintf(str_goto,"%s F%.3lf",d_varpinch);
			PTP(add_cls(add_feedrate(str_goto),nc_block_in));
		}
		Xold=Xout; Yold=Yout; type_of_goto=-1; /* save what you need */
		rapid_mod=0;
	} 
/* CIRCLE */
	else if((s1 = strstr(nc_block_in, circle)) != NULL)	
	{
/* check is the FROM command in operation ? */
   if(!from_move && mill_mode && !meas_tool) 
   { 
      uc1601("ERROR: CSL operation MUST include FROM point", 1);
      uc4540(chanel_out, 0);
	  strcat (strcpy(nc_block_out,"rm "),ptp_file);
	  system(nc_block_out);
	  break;
   }
   sscanf(s1+7,"%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf",
          &tmp1,&tmp2,&tmp3,&tmp1,&tmp2,&tmp3,&R,&tolerance,&ratio);
   XYZ_input(s1+7,&Xc,&Yc,&Zc,&Nx,&Ny,&Nz,mill_mode);
   uc4514(chanel_in, str_goto); /* read a next GOTO command */
   XYZ_input(str_goto+5,&Xe,&Ye,&Ze,&tmp1,&tmp2,&tmp3,mill_mode);
   angle=acos( ((Xout-Xc)*(Xe-Xc)+(Yout-Yc)*(Ye-Yc)+(Zout-Zc)*(Ze-Zc))/(R*R));
   circle_unite(Xc,Yc,Zc,Nx,Ny,Nz,R, &Xe,&Ye,&Ze,&angle); 
   if(mill_mode && !tool_sys)
   {
      Br=coord_b1/180.0*M_PI;
      C_rotate(&Ye,&Ze,Cr); C_rotate(&Yc,&Zc,Cr);  C_rotate(&Ny,&Nz,Cr);
      Xe=around(Xe); Ye=around(Ye); Ze=around(Ze);
      Xc=around(Xc); Yc=around(Yc); Zc=around(Zc);
      circle_process_mill(coord_b1, Xout,Yout,Zout, Xc,Yc,Zc, Nx,Ny,Nz, Xe,Ye,Ze,
                          R,tolerance,ratio,angle );
	  Xold=Xe; Yold=Ye; Zold=Ze; Xout=Xe; Yout=Ye; Zout=Ze;	/* save what we need */
   }
   else if(mill_mode && tool_sys)
   {
      Br=coord_b1/180.0*M_PI;
	  Yc_cls=Yc; Zc_cls=Zc;
      C_rotate(&Ye,&Ze,Cr); C_rotate(&Yc,&Zc,Cr);  C_rotate(&Ny,&Nz,Cr);
      B_rotate(&Xe,&Ze,Xoff,Zoff,Br);
      B_rotate(&Xc,&Zc,Xoff,Zoff,Br);
      B_rotate(&Nx,&Nz,0.0,0.0,Br);
      Xe=around(Xe); Ye=around(Ye); Ze=around(Ze);
      Xc=around(Xc); Yc=around(Yc); Zc=around(Zc);
      if(fabs(Nz) - 1.0 > 0.00001)
      { 
         uc1601("ERROR: Normal of Circle doesnot match Tool axis", 1);
         uc4540(chanel_out, 0);
		 strcat (strcpy(nc_block_out,"rm "),ptp_file);
		 system(nc_block_out);
		 break;
	  }
      if(Nz > 0 && type_of_goto != 2) 
      { 
         strcpy(COMMAND," G2");	
         type_of_goto=2; 
      }
      else if(Nz < 0 && type_of_goto != 3) 
      { 
         strcpy(COMMAND," G3");	
         type_of_goto=3; 
      }
      else strcpy(COMMAND,"");
      sprintf(nc_block_out, "%s E%.3lf Y%.3lf I%.3lf J%.3lf",
                            COMMAND, Xe, Ye, Xc-Xout, Yc-Yout);
      PTP(add_cls(add_feedrate(nc_block_out),nc_block_in));
      Xold=Xe; Yold=Ye; Zold=Ze;		
      Xout=Xe; Yout=Ye; Zout=Ze;	/* save what we need */
   }
   else
   {
      if(Nz < 0.0 && type_of_goto != 3) 
      { 
         strcpy(COMMAND," G3"); 
         type_of_goto=3; 
      }
      else if(Nz >= 0.0 && type_of_goto != 2) 
      {	
         strcpy(COMMAND," G2"); 
         type_of_goto=2;	
      }
      else strcpy(COMMAND,""); 
      sprintf(nc_block_out, "%s X1=%.3lf Z1=%.3lf I%.3lf K%.3lf", 
                             COMMAND, Ye*2, Xe, Yc-Y, Xc-X);
      PTP(add_cls(add_feedrate(nc_block_out),nc_block_in));
      Xold=Xe; Yold=Ye; Zold=Ze;		
      X=Xe; Y=Ye; Z=Ze;	/* save what we need */
   }
}/* endof CIRCLE */
/* L843 */
	else if((s1 = strstr(nc_block_in, l843)) != NULL)
	{
		uc4514(chanel_in,nc_block_in);
		if( strstr(nc_block_in,"RAPID")!=NULL) uc4514(chanel_in,nc_block_in);
		if( strstr(nc_block_in,"GOTO") ==NULL) 
		{ 
			uc1601("ERROR: L843 command does not have GOTO for origin set", 1);
			uc4540(chanel_out, 0);
			strcat (strcpy(nc_block_out,"rm "),ptp_file);
			system(nc_block_out);
			break;
		}  
		Xold=Yold=Zold=-999999.999;	/* reset of coordinat modality */
		XYZ_input(nc_block_in+5,&Xout,&Yout,&Zout,&tmp1,&tmp2,&tmp3,mill_mode);
		C_rotate(&Yout,&Zout,Cr);
		Xoff=Xout; Zoff=Zout;          /* save the shift for B rotataion transform */
		sprintf(nc_block_out," G58 E1=%.3lf Z1=%.3lf",Zout,Xout);  
		PTP(add_cls(add_feedrate(nc_block_out),nc_block_in));      
		PTP(" L843");
		Br=coord_b1/180.0*M_PI;	
		tool_sys=1;
		type_of_goto=-1;
	}
/* L844 */   
	else if((s1 = strstr(nc_block_in, "L844")) != NULL)
	{
		PTP(" L844"); PTP(" G58 E1=0 Z1=0");
		Xold=Yold=Zold=-999999.999;	/* reset of coordinat madality */
		tool_sys=0;
		plane_of_circle=type_of_goto=-1;
	}
/* END OF PATH */   
	else if((s1 =strstr(nc_block_in,"END-OF-PATH"))!=NULL) 
	{
		from_move=upcom=0; type_of_goto=-1;	
		Xold=Yold=Zold=Cold=coord_b1=-999999.99;
	}
/* M13 */   
	else if ((s1 = strstr(nc_block_in, m13))!=NULL)  f_m13();
/* M14 */   
	else if ((s1 = strstr(nc_block_in, m14))!=NULL)  f_m14();
/* M19 */   
	else if ((s1 = strstr(nc_block_in, m19))!=NULL)  f_m19(mill_mode);
/* M56 */   
	else if ((s1 = strstr(nc_block_in, m56))!=NULL)  f_m56();
/* M57 */   
	else if ((s1 = strstr(nc_block_in, m57))!=NULL)  f_m57();
/* L822 */   
	else if ((s1 = strstr(nc_block_in, l822))!=NULL)  { f_l822(); reset_XYZC(); }
/* L823 */   
	else if ((s1 = strstr(nc_block_in, l823))!=NULL)  { f_l823(); reset_XYZC(); }
/* L824 */   
	else if ((s1 = strstr(nc_block_in, l824))!=NULL)  { f_l824(); reset_XYZC(); }
/* L825 */   
	else if ((s1 = strstr(nc_block_in, l825))!=NULL)  { f_l825(); reset_XYZC(); }
/* L827 */   
	else if ((s1 = strstr(nc_block_in, l827))!=NULL)  { f_l827(); reset_XYZC(); }
/* L828 */  
	else if ((s1 = strstr(nc_block_in, l828))!=NULL)  { f_l828(); reset_XYZC(); }
/* L830 */   
	else if ((s1 = strstr(nc_block_in, l830))!=NULL)  { f_l830(); reset_XYZC(); }
/* L831 */   
	else if ((s1 = strstr(nc_block_in, l831))!=NULL)  { f_l831(); reset_XYZC(); }
/* L832 */   
	else if ((s1 = strstr(nc_block_in, l832))!=NULL)  { f_l832(); reset_XYZC(); }
/* L833 */   
	else if ((s1 = strstr(nc_block_in,  l833))!=NULL) { f_l833(); reset_XYZC(); }
/* L834 */   
	else if ((s1 = strstr(nc_block_in, l834))!=NULL)  { f_l834(); reset_XYZC(); }
	} 	
/* ====================> End while statement <================================*/
	PTP(" T1=0 R1=1 R3=0 R4=0 R6=0 L106");
	PTP(" T1=0");
	PTP(" M30");
	uc4540(chanel_in, 0);
	uc4540(chanel_out, 0);
	filter = "*.prt";
	UF_UI_set_dialog_filter(UF_UI_PART_NEW_FLTR, filter);
	uc1601("Post Processing Finished", 1);
/*	}
*/
	UF_terminate();  
}

/* End of main program */

/* Function unload shared image from memory immediatly */

int ufusr_ask_unload(void)
{
  return (UF_UNLOAD_IMMEDIATELY);
}

/* Function read parameters of tool */

int f_tldata(char *str_1)
{
   char *mill = "MILL", *drill = "DRILL", *turn = "TURN", *lathe="LATHE"; 
   char *groove = "GROOVE", *thread = "THREAD", *button = "BUTTON";
   char *barrel = "BARREL", *tcutter = "TCUTTER", *borbar = "BORBAR";
   char *buttress = "BUTTRESS", *acme = "ACME", *ring = "RING";
   char *left = "LEFT", *rigth = "RIGTH", *s1;
   int type;
   if((s1 = strstr(str_1, drill)) == NULL && 
      (s1 = strstr(str_1, mill)) != NULL) 
   {
      sscanf(s1+5, "%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf", &mill_tool.diameter,
      &mill_tool.low_radius, &mill_tool.length, &mill_tool.taper_angle, 
      &mill_tool.tip_angle, &mill_tool.x_center_r1, &mill_tool.y_center_r1, 
      &mill_tool.upper_radius, &mill_tool.x_center_r2, &mill_tool.y_center_r2); 
      type = 1;
   }
   else if((s1 = strstr(str_1, barrel)) != NULL) 
   {
      sscanf(s1+7, "%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf", &barrel_tool.diameter,
      &barrel_tool.bar_radius, &barrel_tool.low_radius, &barrel_tool.upp_radius,
      &barrel_tool.length, &barrel_tool.flute_length, &barrel_tool.y_center_r1,
      &barrel_tool.shank_diameter);
      type = 2;
   }
   else if((s1 = strstr(str_1, tcutter)) != NULL) 
   {
      sscanf(s1+8, "%lf,%lf,%lf,%lf,%lf,%lf", &tcutter_tool.diameter,
      &tcutter_tool.low_radius, &tcutter_tool.upp_radius,
      &tcutter_tool.length, &tcutter_tool.flute_length, &tcutter_tool.shank_diameter);
      type = 3;
   }
   else if((s1 = strstr(str_1, mill)) != NULL && 
           (s1 = strstr(str_1, drill)) != NULL) 
   {
      s1 = strstr(str_1, drill);
      sscanf(s1+11, "%lf,%lf,%lf,%lf,%lf", &drill_tool.diameter,
      &drill_tool.corner_radius, &drill_tool.length, &drill_tool.point_angle, 
      &drill_tool.flute_length); 
      type = 4;
   }
   else if((s1 = strstr(str_1, drill)) != NULL && 
           (s1 = strstr(str_1, lathe)) != NULL) 
   {
      s1 = strstr(str_1, drill);
      sscanf(s1+11, "%lf,%lf,%lf,%lf,%lf", &drill_tool.diameter,
      &drill_tool.corner_radius, &drill_tool.length, &drill_tool.point_angle, 
      &drill_tool.flute_length); 
      type = 11;
   }
   else if((s1 = strstr(str_1, turn)) != NULL) 
   {
      if((s1 = strstr(str_1, button)) != NULL) 
      {
         sscanf(s1+7, "%6s%8s%lf,%lf,%lf,%lf", button_tool.tool_orient, 
         button_tool.type_of_work, &button_tool.noze_diam, &button_tool.holder_width,
         &button_tool.holder_heigth, &button_tool.orient_angle);
         type = 6;
      }
      else if((s1 = strstr(str_1, borbar)) != NULL) 
      {
         sscanf(s1+7, "%6s%8s%lf,%lf,%lf,%lf,%lf,%lf,%lf", bar_tool.tool_orient, 
         bar_tool.type_of_work, &bar_tool.noze_diam, &bar_tool.holder_width, 
         &bar_tool.lead_angle, &bar_tool.heel_angle, &bar_tool.bar_radius, 
         &bar_tool.holder_heigth, &bar_tool.orient_angle);
         type = 7;
      }
      else
      {
         s1 = strstr(str_1, turn);
         sscanf(s1+5, "%6s%8s%lf,%lf,%lf,%lf,%lf,%lf", turn_tool.tool_orient, 
         turn_tool.type_of_work, &turn_tool.noze_diam, &turn_tool.holder_width, 
         &turn_tool.lead_angle, &turn_tool.heel_angle, &turn_tool.holder_heigth, 
         &turn_tool.orient_angle);
         type = 5;
      }	 
   }	 
   else if((s1 = strstr(str_1, groove)) != NULL) 
   {
      if((s1 = strstr(str_1, ring)) != NULL)
      {
         sscanf(s1+5, "%5s%8s%4s%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf", 
         angle_groove_tool.tool_orient, angle_groove_tool.type_of_work, 
         angle_groove_tool.trace_size, &angle_groove_tool.insert_width,
         &angle_groove_tool.left_angle, &angle_groove_tool.rigth_angle, 
         &angle_groove_tool.max_depth, &angle_groove_tool.lead_angle, 
         &angle_groove_tool.insert_angle, &angle_groove_tool.holder_height,
         &angle_groove_tool.holder_width);
         type = 9;
      }
      else
      {
         s1 = strstr(str_1, groove);
         sscanf(s1+7, "%5s%8s%4s%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf", groove_tool.tool_orient, 
         groove_tool.type_of_work, groove_tool.trace_size, &groove_tool.insert_width,
         &groove_tool.left_radius, &groove_tool.rigth_radius, &groove_tool.max_depth,
         &groove_tool.lead_angle, &groove_tool.insert_angle, &groove_tool.holder_height,
         &groove_tool.holder_width);
      }
      type = 8;
   }
   else if((s1 = strstr(str_1, thread)) != NULL) 
   {
      if((s1 = strstr(str_1, acme)) != NULL) 
      {
         sscanf(s1+5, "%5s%8s%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf", acme_thread_tool.tool_orient, 
         acme_thread_tool.type_of_work, &acme_thread_tool.max_depth, 
         &acme_thread_tool.holder_width, &acme_thread_tool.refief_angle, 
         &acme_thread_tool.heel_angle, &acme_thread_tool.nose_width,
         &acme_thread_tool.holder_height, &acme_thread_tool.insert_width, 
         &acme_thread_tool.tip_offset);
      }
      else if((s1 = strstr(str_1, buttress)) != NULL)
      {
         sscanf(s1+9, "%5s%8s%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf", but_thread_tool.tool_orient, 
         but_thread_tool.type_of_work, &but_thread_tool.max_depth, 
         &but_thread_tool.holder_width, &but_thread_tool.refief_angle, 
         &but_thread_tool.heel_angle, &but_thread_tool.nose_radius, 
         &but_thread_tool.holder_height, &but_thread_tool.insert_width, 
         &but_thread_tool.tip_offset);
      }
      else
      {
	 s1 = strstr(str_1, thread);
         sscanf(s1+7, "%5s%8s%lf,%lf,%lf,%lf,%lf,%lf,%lf", thread_tool.tool_orient, 
         thread_tool.type_of_work, &thread_tool.max_depth, &thread_tool.holder_width,
         &thread_tool.refief_angle, &thread_tool.heel_angle, &thread_tool.holder_height,
         &thread_tool.insert_width, &thread_tool.tip_offset);
      }
      type = 10;
   }      				 
   else uc1601("Unknow (or new) tool type", 1);
   return type;
}
                           /* ALL FUNNCTIOn WHICH HAVE A NAME f_... */
/* Coolant on/off commands */
void f_coolnt(char *str_1)
{
	char *on = "ON", *off = "OFF", *mist = "MIST", *flood = "FLOOD", *s1;	
	if((s1 = strstr(str_1, on)) != NULL)	        PTP(" M3=8");
	else if((s1 = strstr(str_1, off)) != NULL)		 PTP("	M3=9");
	else if((s1 = strstr(str_1, mist)) != NULL)	 PTP(" M3=7");
	else if((s1 = strstr(str_1, flood)) != NULL)	PTP(" M99" );
}

/* Delay command */

void f_delay(char *str_1)
{
	double d_delay, d_tmp;
	int prec = 0;
	sscanf(str_1+6, "%lf", &d_delay);
	if(modf(d_delay, &d_tmp) == 0.0) prec = 0;
	else prec = 3;
	sprintf(nc_block_out, " G4 X%.*lf", prec, d_delay);
}

/* LOAD TOOL command */

int f_load_tool(double *coord_b1, int multi_axis, char *str_1)
{
   int tool_num;
   (*coord_b1)=around(*coord_b1);
   sscanf(str_1+10, "%d", &tool_num);
   if(multi_axis == 1)
   {
      sprintf(nc_block_out, " T1=%d R1=1 R3=0 R4=0 R6=0 L106", tool_num);
      PTP(nc_block_out);
      PTP(" @36a R52 K2        ( Read actual D-No )");
      PTP(" @303 R53 K1 K4000  ( Read swivek-dimension )");
      PTP(" @320 R54 K1 R52 K2 ( Read tool L1 - geometry )");
      PTP(" @320 R55 K1 R52 K5 ( Read tool L1 wcar )");
      PTP(" R51=R53+R54+R55    ( Total Lenght of tool )");
      PTP(" M6=150");
      PTP(" M3=161 H3=1");
      PTP(" D0");
   }
   else
   {
      sprintf(nc_block_out, " T1=%d R1=1 R3=0 R4=%.3lf R6=0 L106", tool_num, *coord_b1);
      PTP(nc_block_out);
      if(fabs(fmod(*coord_b1,2.5)) > 0.01) PTP(" M6=152"); 
      else PTP(" M6=151");
  /* Process monitoring -> Not using for measuring and thread tools !!! */			
		/*			
						sprintf(nc_block_out, " M3=161 H3=%d", tool_change);
						PTP(nc_block_out);	
						if(tool_change > 200) tool_change = 1;
      tool_change++;
  */
   }
   if (tool_num > 99999990) return (1);
   else                     return (0);
}

/* CUTCOM command */

int f_cutcom(char *str_1)
{
   char *on = "ON", *off = "OFF", *left = "LEFT", *right = "RIGHT", *s1;
   int comp;
   if     ((s1 = strstr(str_1, on)) != NULL)    comp = 42;
   else if((s1 = strstr(str_1, off)) != NULL)   comp = 40;
   else if((s1 = strstr(str_1, left)) != NULL)  comp = 41;
   else if((s1 = strstr(str_1, right)) != NULL) comp = 42;
   return comp;
}

/* SPINDEL ON/OFF command */

void f_spindl( int chanel_in, int mill_mode, char *str_1)
{
   char *s1, str_2[256], *rpm = "RPM", *smm = "SMM", *maxrpm = "MAXRPM"; 
   char *clw = "CLW", *cclw = "CCLW", *range = "RANGE", *off = "OFF";
   int i_range, i_rpm, i_smm, m1, mode = 95, file_pos;
   int max_rpm, in;
   if((s1 = strstr(str_1, clw)) != NULL) m1 = 3;
   if((s1 = strstr(str_1, cclw)) != NULL) m1 = 4;
   if((s1 = strstr(str_1, maxrpm)) != NULL)
   {
      sscanf(s1+7, "%d", &max_rpm);
      sprintf(nc_block_out, " G92 S1=%d", max_rpm);	PTP(nc_block_out);
   }
   if((s1 = strstr(str_1, range)) != NULL)
   {
      sscanf(s1+6, "%d", &i_range);
      sprintf(nc_block_out, " M1=4%d", i_range);	PTP(nc_block_out);
   }
   if((s1 = strstr(str_1, smm)) != NULL)
   {
      sscanf(s1+4, "%d", &i_smm);
						PTP(" M1=61");
      sprintf(nc_block_out, " G96 S1=%d M1=%d", i_smm, m1);	PTP(nc_block_out);
   }
   else if((s1 = strstr(str_1, rpm)) != NULL)
   {
      sscanf(s1+4, "%d", &i_rpm);
      file_pos = uc4534(chanel_in);
      while((in = uc4514(chanel_in, str_2) >= 0))
      {
         if((s1 = strstr(str_2, "MMPM")) != NULL) 
         {
            mode = 94;
	        break;
	     }
	     else if((s1 = strstr(str_2, "MMPR")) != NULL)
	     {   
	        mode = 95;
            break;
	     }
      }
      uc4535(chanel_in, file_pos);
      if(mill_mode == 0)
      {
         PTP(" M1=61");
         sprintf(nc_block_out, " G%d S1=%d M1=%d", mode, i_rpm, m1); PTP(nc_block_out);
      }
      else
      {
         PTP(" M3=61");
         sprintf(nc_block_out, " G%d S3=%d M3=%d", mode, i_rpm, m1); PTP(nc_block_out);
      }
   }
   else if((s1 = strstr(str_1, off)) != NULL)
   {
      if(mill_mode == 0) PTP(" M1=5");
      else PTP(" M3=5");
   }
}

/* GOHOME command */

void f_gohome( int mill_mode, char *str_1)
{  
   double x, y, z;
   XYZ_input(str_1+7, &x, &y, &z, &tmp1,&tmp2,&tmp3,mill_mode);
   x=around(x); y=around(y); z=around(z); 
   if(mill_mode == 1)	sprintf(nc_block_out, " G53 G0 D0 E1=%.3lf Y1=%.3lf Z1=%.3lf", z, y, x);
   else					sprintf(nc_block_out, " G53 G0 D0 X1=%.3lf Z1=%.3lf"         , y*2,  x);
   PTP(nc_block_out);
}
/* Include file measur.h for created measuring cycle */
/* Measuring cycle definition */
void f_l822() 
{
  double position[3] = {0.0, 0.0, 0.0}, position1[3] = {0.0, 0.0, 0.0};
  char *message = "Select point to prepare position";
  char menu_list[6][38] = {"X1 axis", "E1 axis", "Z1 axis", "Y1 axis", 
  "C1 axis", "B1 axis"};
  char menu_list_1[3][16]={"R2","R3","R4"};
  int mes_axis, type_items[3]={200, 100, 100};
  int points[3]={1,1,1}, mode[2] = {6, 0}, irc, type_item_d[1]={200};
  double input_d[3] = {10.0, 0.0, 0.0}, degree_tool=0.0;
  char inp_values[3][31] = {1,1,1};
  char menu_list_21[1][16]={"Angle of B axis"};
  char *mes_tool = "Input angle (-105...90) of orientation measuring tool";
  
  message = "Select axis for measuring"; 
  irc = uc1603(message, 1, menu_list, 6);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
  switch(irc)
  {
     case 2:  exit(0);				break;
     case 5:  mes_axis = 1;			break;
     case 6:  mes_axis = 4;			break;
     case 7:  mes_axis = 2;			break;
     case 8:  mes_axis = 5;			break;
     case 9:  mes_axis = 3;			break;
     case 10: mes_axis = 6;		    break;
     default: mes_axis = 4;
  }
  message = "Input parameters measuring cycle L822";
  irc = uc1613(message, menu_list_1, 3, points, input_d, 
  inp_values, type_items);
  if(irc == 2) exit(0);
  around(input_d[0]);
  sprintf(nc_block_out, " R1=%d R2=%.3lf R3=%d R4=%d L822",mes_axis, input_d[0], points[1], points[2]); 
  PTP(nc_block_out);
  return	;
}

/* Create cycle for Search a characteristic */

void f_l823()
{
   double position[3] = {0.0, 0.0, 0.0}, position1[3] = {0.0, 0.0, 0.0};
   char *message = "Select point to prepare position";
   char menu_list_21[1][16]={"Angle of B axis"};
   char menu_list[6][38] = {"X1 axis", "E1 axis", "Z1 axis", "Y1 axis", 
   "C1 axis", "B1 axis"};
   char menu_list_1[4][16]={"R2","R4","R5","R6"};
   int mes_axis, search_axis, type_items[4]={200, 200, 100, 100};
   int points[4]={1,1,1,1}, mode[2] = {6, 0}, irc, type_item_d[1]={200};
   double input_d[4] = {10.0, 10.0, 0.0, 0.0}, degree_tool=0.0;
   char inp_values[4][31] = {1,1,1,1};
   char *mes_tool = "Input angle (-105...90) of orientation measuring tool";
  
   message = "Select axis for measuring"; 
   irc = uc1603(message, 1, menu_list, 6);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
   switch(irc)
   {
      case 5 : mes_axis = 1;	break;
      case 6 : mes_axis = 4;	break;
      case 7 : mes_axis = 2;	break;
      case 8 : mes_axis = 5;	break;
      case 9 : mes_axis = 3;	break;
      case 10: mes_axis = 6;    break;
      default: mes_axis = 4;
    }
    message = "Select searching axis";
    irc = uc1603(message, 1, menu_list, 6);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
    switch(irc)
    {
      case  5: search_axis = 1;	break;
      case  6: search_axis = 4;	break;
      case  7: search_axis = 2;	break;
      case  8: search_axis = 5;	break;
      case  9: search_axis = 3;	break;
      case 10: search_axis = 6;	break;
      default: search_axis = 4;
   }
   message = "Input parameters measuring cycle L823";
   irc = uc1613(message, menu_list_1, 4, points, input_d, 
   inp_values, type_items);
   sprintf(nc_block_out, " R1=%d R2=%.3lf R3=%d R4=%.3lf R5=%d R6=%d L823",
   mes_axis, input_d[0], search_axis, input_d[1], points[2], points[3]);
   PTP(nc_block_out);
}

/* Create cycle for testing a characteristic */

void f_l824()
{
    double position[3] = {0.0, 0.0, 0.0}, position1[3] = {0.0, 0.0, 0.0};
    char *message = "Select point to prepare position";
    char menu_list[6][38] = {"X1 axis", "E1 axis", "Z1 axis", "Y1 axis", 
    "C1 axis", "B1 axis"};
    char menu_list_1[2][16]={"R2","R3"};
    int mes_axis, type_items[2]={200, 100}, type_item_d[1]={200};
    int points[2]={1,1}, mode[2] = {6, 0}, irc;
    double input_d[2] = {10.0, 0.0}, degree_tool=0.0;
    char inp_values[2][31] = {1,1};
    char menu_list_21[1][16]={"Angle of B axis"};	
    char *mes_tool = "Input angle (-105...90) of orientation measuring tool";
    message = "Select axis for measuring"; 
    irc = uc1603(message, 1, menu_list, 6);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
    switch(irc)
    {
       case  5: mes_axis = 1;	break;
       case  6: mes_axis = 4;	break;
       case  7: mes_axis = 2;	break;
       case  8: mes_axis = 5;	break;
       case  9: mes_axis = 3;	break;
       case 10: mes_axis = 6;	break;
       default: mes_axis = 4;
   }
   message = "Input parameters measuring cycle L823";
   irc = uc1613(message, menu_list_1, 2, points, input_d, inp_values, type_items);
   input_d[0]=around(input_d[0]);
   sprintf(nc_block_out, " R1=%d R2=%.3lf R3=%d L824", mes_axis, input_d[0], points[1]);
   PTP(nc_block_out);
   return ;
}

/* Create cycle for measuring diameter to 2 points */

void f_l825()
{
   double position[3] = {0.0, 0.0, 0.0}, position1[3] = {0.0, 0.0, 0.0};
   double position2[3] = {0.0, 0.0, 0.0}, degree_tool=0.0;
   int irc, mode[2] = {6, 0}, mes_axis, search_axis;
   int type_item_d[1]={200};
   char inp_values[1][31]={1};
   char menu_list_2[1][16]={"Angle of B axis"};
   char *message = "Select point to prepare position";
   char *mes_sel_axis = "Select measuring axis";
   char menu_list[6][38] = {"X1 axis", "E1 axis", "Z1 axis", "Y1 axis", 
   "C1 axis", "B1 axis"};
   char menu_list_825[13][16]={"R2", "R3", "R4", "R6", "R7", "R11", 
   "R21", "R22", "R23", "R25", "R26", "R27", "R28"};
   int type_item_825[13] = {200,200,100,200,200,200,100,100,100,200,200,200,200};
   int input_int_825[13] = {1,1,1,1,1,1,1,1,1,1,1,1,1};
   char input_char_825[13][31] = {"1","2","3","4","5","6",
   "8","9","10","11","12","13","14"}; 
   double input_double_825[13] = {252.6,0.0,0.0,30.0,2.0,0.0,
   0.0,0.0,0.0,0.016,-0.016,1.0,0.2};
   char *mes_tool = "Input angle (-105...90) of orientation measuring tool";
   int i;

   irc = uc1603(mes_sel_axis, 1, menu_list, 6);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
   switch(irc)
   {
      case  5: mes_axis = 1;	break;
      case  6: mes_axis = 4;	break;
      case  7: mes_axis = 2;	break;
      case  8: mes_axis = 5;	break;
      case  9: mes_axis = 3;	break;
      case 10: mes_axis = 6;	break;
      default: mes_axis = 4;
   }
   mes_sel_axis = "Select compensation axis";
   irc = uc1603(mes_sel_axis, 1, menu_list, 6);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
   switch(irc)
   {
      case  5: search_axis = 1;	break;
      case  6: search_axis = 4; break;
      case  7: search_axis = 2;	break;
      case  8: search_axis = 5; break;
      case  9: search_axis = 3; break;
      case 10: search_axis = 6;	break;
     default : search_axis = 4;	
              
   }
   mes_sel_axis = "Input parameters of measuring cycle L825";
   irc = uc1613(mes_sel_axis, menu_list_825, 13, input_int_825, input_double_825, 
   input_char_825, type_item_825);
   for (i=0;i<13;i++) input_double_825[i]=around(input_double_825[i]);
   sprintf(nc_block_out, " R1=%d R2=%.3lf R3=%.3lf R4=%d R5=%d R6=%.3lf R7=%.3lf R11=%.3lf\
    \nR21=%d R22=%d R23=%d R25=%.3lf R26=%.3lf R27=%.3lf R28=%.3lf L825",
   mes_axis, input_double_825[0], input_double_825[1], input_int_825[2], search_axis, input_double_825[3], input_double_825[4], 
   input_double_825[5], input_int_825[6], input_int_825[7], input_int_825[8], 
   input_double_825[9], input_double_825[10], input_double_825[11], input_double_825[12]);
   PTP(nc_block_out);

   return ;
}

/* Create cycle for measuring angle */

void f_l827()
{
   double position[3] = {0.0, 0.0, 0.0}, position1[3] = {0.0, 0.0, 0.0};
   char *message = "Select point to prepare position";
   char menu_list[6][38] = {"X1 axis", "E1 axis", "Z1 axis", "Y1 axis", 
   "C1 axis", "B1 axis"};
   char menu_list_1[3][16]={"R2","R3","R5"};
   int mes_axis, search_axis, type_items[3]={200, 100, 200};
   int points[3]={1,1,1}, mode[2] = {6, 0}, irc, type_item_d[1]={200};
   double input_d[3] = {10.0, 0.0, 10.0}, degree_tool=0.0;
   char inp_values[3][31] = {1,1,1};
   char menu_list_21[1][16]={"Angle of B axis"};
   char *mes_tool = "Input angle (-105...90) of orientation measuring tool";

   message = "Select axis for measuring"; 
   irc = uc1603(message, 1, menu_list, 6);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
   switch(irc)
   {
      case  5: mes_axis = 1;	break;
      case  6: mes_axis = 4;    break;
      case  7: mes_axis = 2;	break;
      case  8: mes_axis = 5;	break;
      case  9: mes_axis = 3;	break;
      case 10: mes_axis = 6;	break;
      default: mes_axis = 4;
   }
   message = "Select searching axis";
   irc = uc1603(message, 1, menu_list, 6);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
   switch(irc)
   {
     case  5: search_axis = 1;	break;
     case  6: search_axis = 4;	break;
     case  7: search_axis = 2;	break;
     case  8: search_axis = 5;	break;
     case  9: search_axis = 3;	break;
     case 10: search_axis = 6;	break;
     default: search_axis = 4;	break;
   }
   message = "Input parameters measuring cycle L827";
   irc = uc1613(message, menu_list_1, 3, points, input_d, inp_values, type_items);    
   input_d[0]=around(input_d[0]);
   input_d[2]=around(input_d[2]);
   sprintf(nc_block_out, "R1=%d R2=%.3lf R3=%d R4=%d R5=%.3lf L827", mes_axis, input_d[0], points[1], search_axis, input_d[2]);
   PTP(nc_block_out); 
   return;
}

/* Create cycle L828 for measure true running */

void f_l828()
{
   double position[3] = {0.0, 0.0, 0.0}, position1[3] = {0.0, 0.0, 0.0};
   char *message = "Select point to prepare position";
   char menu_list[6][38] = {"X1 axis", "E1 axis", "Z1 axis", "Y1 axis", 
   "C1 axis", "B1 axis"};
   char menu_list_1[4][16]={"R2","R3","R5","R6"};
   int mes_axis, type_items[4]={200, 100, 200, 100};
   int points[4]={1,1,1,1}, mode[2] = {6, 0}, irc, type_item_d[1]={200};
   double input_d[4] = {10.0, 0.0, 0.0, 0.0}, degree_tool=0.0;
   char inp_values[4][31] = {1,1,1,1};   
   char menu_list_21[1][16]={"Angle of B axis"};
   char *mes_tool = "Input angle (-105...90) of orientation measuring tool";
   
   message = "Select axis for measuring"; 
   irc = uc1603(message, 1, menu_list, 6);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
   switch(irc)
   {
      case  5: mes_axis = 1;	break;
      case  6: mes_axis = 4;	break;
      case  7: mes_axis = 2;	break;
      case  8: mes_axis = 5;	break;
      case  9: mes_axis = 3;	break;
      case 10: mes_axis = 6;	break;
      default: mes_axis = 4;
   }
   message = "Input parameters measuring cycle L828";
   irc = uc1613(message, menu_list_1, 4, points, input_d, 
   inp_values, type_items);    
   input_d[0]=around(input_d[0]);
   input_d[2]=around(input_d[2]);
   sprintf(nc_block_out, " R1=%d R2=%.3lf R3=%d R4=3 R5=%.3lf R6 L828", mes_axis, input_d[0], points[1], input_d[2]);
   PTP(nc_block_out); 
   return ;
}

/* Create cycle for calculate values */

void f_l830()
{
   char menu_list_830[3][16]={"R1", "R2", "R3"};
   char input_char_830[3][31]={"1","2","3"};
   int input_int_830[3]={1, 2, 1};
   int type_item_830[3]={100,100,100};
   double input_double_830[3]={0.0, 0.0, 0.0};
   int irc;
   char *message = "Input parameters of measuring cycle L830";
   irc = uc1613(message, menu_list_830, 3, input_int_830, input_double_830, 
   input_char_830, type_item_830);
   sprintf(nc_block_out, " R1=%d R2=%d R3=%d L830", input_int_830[0], input_int_830[1], input_int_830[2]);
   PTP(nc_block_out);
   return ;
}

/* Create cycle for write parameters in memory */

void f_l831()
{
   char menu_list_831[9][16]={"R1","R2","R3","R4","R5","R6","R7","R8","R11"};
   char input_char_831[9][31]={"1", "2", "3", "4", "5", "6", "7", "8", "9"};
   int input_int_831[9]={1, 1, 1, 4, 5, 6, 7, 8, 9};
   int type_item_831[9]={100,100,100,200,200,200,200,200,200};
   double input_double_831[9]={0.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0};
   int irc, i;
   char *mes_sel_axis = "Input parameters of measuring cycle L831";
   irc = uc1613(mes_sel_axis, menu_list_831, 9, input_int_831, input_double_831,
   input_char_831, type_item_831); 

   for(i = 3; i < 9; i++) input_double_831[i] = around(input_double_831[i]);

   sprintf(nc_block_out, " R1=%d R2=%d R3=%d R4=%.3lf R5=%.3lf R6=%.3lf R7=%.3lf R8=%.3lf R11=%.3lf L831",
   input_int_831[0], input_int_831[1], input_int_831[2],	input_double_831[3], input_double_831[4],  
   input_double_831[5], input_double_831[6], input_double_831[7], input_double_831[8]);
   PTP(nc_block_out);
   return ;
}

/* Create cycle for Entry in zero shift */

void f_l832()
{
   double degree_tool=0.0;
   int type_item_d[1]={200};
   char menu_list_21[1][16]={"Angle of B axis"};
   int zero_shift, mes_axis, type_of_entry, irc;
   char inp_values[1][31]={1};
   char *message = "Select zero shift";
   char *mes_sel_axis; 
   char menu_list[6][38] = {"X1 axis", "E1 axis", "Z1 axis", "Y1 axis", 
   "C1 axis", "B1 axis"};
   char menu_list_10[4][38]={"G54", "G55", "G56", "G57"};
   char menu_list_11[4][38]={"Write rough shift", "Write fine shift", 
   "Add to rough shift", "Add to fine shift"};
   char *mes_tool = "Input angle (-105...90) of orientation measuring tool";

   irc = uc1603(message, 1, menu_list_10, 4);	
   if(irc == i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
   switch(irc)
   {
      case  5: zero_shift = 54;	break;
      case  6: zero_shift = 55; break;
      case  7: zero_shift = 56; break;
      case  8: zero_shift = 57;	break;
      default: zero_shift = 54;	
   }
   mes_sel_axis = "Select measuring axis"; 
   irc = uc1603(mes_sel_axis, 1, menu_list, 6);	
   if(irc == i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
   switch(irc)
   {
      case  5: mes_axis = 1;	break;
      case  6: mes_axis = 4;	break;
      case  7: mes_axis = 2;	break;
      case  8: mes_axis = 5;	break;
      case  9: mes_axis = 3;	break;
      case 10: mes_axis = 6;	break;
      default: mes_axis = 4;
   }
   mes_sel_axis = "Select type of entry"; 
   irc = uc1603(mes_sel_axis, 1, menu_list_11, 4);	if(irc==i_cancel) PTP_CLOSE(" PTP close by CANSEL comand");
   switch(irc)
   {
   case  5: type_of_entry = 1;	break;
   case  6: type_of_entry = 2;	break;
   case  7: type_of_entry = 3;  break;
   case  8: type_of_entry = 4;	break;
   default: type_of_entry = 1;	
   } 
   sprintf(nc_block_out, " R1=%d R2=%d R3=%d L832", zero_shift, mes_axis, type_of_entry);
   PTP(nc_block_out);
   return ;
}

/* Create cycle for write measuring data in hard disk */

void f_l833()
{
   char *message = "Input parameters for measuring cycle L834";
   int irc, points[3] = {1,1,1}, type_item[3] = {100, 100, 100};
   double input_d[3] = {0.0, 0.0, 0.0};
   char menu_list[3][16] = {"R1", "R2", "R3"};
   char inp_values[3][31] = {"1", "1", "1"};
   irc = uc1613(message, menu_list, 3, points, input_d, 
   inp_values, type_item);
   sprintf(nc_block_out, " R1=%d R2=%d R3=%d L833", points[0], points[1], points[2]);
   PTP(nc_block_out);
   return ;
}

/* Create cycle for printing measuring data in printer */

void f_l834()
{
   char *message = "Input parameters for measuring cycle L834";
   int irc, points[3] = {1,1,1}, type_item[3] = {100, 100, 100};
   double input_d[3] = {0.0, 0.0, 0.0};
   char menu_list[3][16] = {"R1", "R2", "R3"};
   char inp_values[3][31] = {"1","1","1"};
   irc = uc1613(message, menu_list, 3, points, input_d, inp_values, type_item);
   sprintf(nc_block_out, " R1=%d R2=%d R3=%d L834", points[0], points[1], points[2]);
   PTP(nc_block_out);
   return ;
}

/* Create cycle position of tailstock */

void f_l930( int zero_shift)
{
   char *message = "Select position of tailstock"; 
   double position[3], position1[3];
   int irc, mode[2] = {6,0};
   irc = uc1616(message, mode, 0, position);
   UF_CSYS_map_point(UF_CSYS_ROOT_COORDS, position, UF_CSYS_ROOT_WCS_COORDS, position1);
   position1[0] = around(position1[0]);

   if(position1[0] == 9999.000) sprintf(nc_block_out, " R1=9999 R2=110 R3=%d L930",  zero_shift);
   else sprintf(nc_block_out, " R1=%.3lf R2=110 R3=%d L930", position1[0]+145.0, zero_shift);
   PTP(nc_block_out);
   return ;
}

/* Create cycle position of lunet */

void f_l931( int zero_shift)
{
   char *message = "Select position of lunet"; 
   double position[3], position1[3];
   int irc, mode[2] = {6,0};
   irc = uc1616(message, mode, 0, position);
   UF_CSYS_map_point(UF_CSYS_ROOT_COORDS, position, UF_CSYS_ROOT_WCS_COORDS, position1);
   
   position1[0] = around(position1[0]);
   if(position1[0] == 9999.000) sprintf(nc_block_out, " R1=9999 R2=0 R3=%d L931" , zero_shift);
   else sprintf(nc_block_out, " R1=%.3lf R2=0 R3=%d L931",	position1[0], zero_shift);
   PTP(nc_block_out);
   return ;
}

/* Create command to close lunet */

void f_m56()
{
   PTP(" M56");	PTP(" G4 X2");
   return ;
}

/* Create command to open lunet */

void f_m57()
{
   PTP(" M57");	PTP(" G4 X2");
   return ;
}

/* Create command to close tailstock */

void f_m13()
{
   PTP(" M13");	PTP(" G4 X2");
   return ;
}

/* Create command to open tailstock */

void f_m14()
{
   PTP(" M14"); PTP(" G4 X2");
   return ;
}

/* Create command to orientation main (turning) spindel */

void f_m19( int mill_mode)
{
   if(mill_mode == 0)	PTP(add_cls(" S1=0 M1=19",nc_block_in));
   else					PTP(add_cls(" S3=0 M3=19",nc_block_in));
   return ;
}

/* Create command for call subprogram to selected tool change 
   position and write subprogram in file SPF9999 */

void f_l9999()
{
   double position[3], position1[3];
   int ch_out_sp, irc,	mode[2] = {6,0};
   char *message = "Select point for tool change position";
   char	*error = "";
   char sp_file[9] = "SPF9999";
   char out_str[128];
   ch_out_sp = uc4504(sp_file, 3, 2004);
   
   if(chanel_out < 0)
   {
      UF_get_fail_message(ch_out_sp, error);
      uc1601(error, 1);
   }			
   irc = uc1616(message, mode, 0, position);
   UF_CSYS_map_point(UF_CSYS_ROOT_COORDS, position, UF_CSYS_ROOT_WCS_COORDS, position1);
   position1[0] = around(position1[0]);
			position1[2] = around(position1[2]);
   sprintf(out_str, "N1 G58 G0 D0 E1=%.3lf Y1=0 Z1=%.3lf", position1[2], position1[0]);   
   uc4524(ch_out_sp,nc_block_out);	
   uc4540(ch_out_sp, 0);
   return ;
}

void position_out(double *position) /* this function output 3 coordinat in PTP file */

{
   position[0]=around(position[0]);
   position[1]=around(position[1]);
   position[2]=around(position[2]);
   
   sprintf(nc_block_out, " G0 E1=%.3lf Y1=%.3lf Z1=%.3lf", 
           position[2], position[1], position[0]);
   PTP(nc_block_out);
}
/* Include file cycle.h for created standart cycles */
void cycle_process(int chanel_in, double b_old,
                   double *x_old, double *y_old, double *z_old, 
                   double *i_old, double *j_old, double *k_old)

{

   char *s1,*s2, s_c[20], s_y[20], COMMAND[4];
    
   char *rapto     = "RAPTO",      s_rapto[20],	  *c_rapto  =" R2=";
   char *fedto     = "FEDTO",      s_fedto[20],   *c_fedto  =" R3=";
   char *rtrcto    = "RTRCTO",     s_rtrcto[20],  *c_rtrcto =" R8=";
   char *mmpm      = "MMPM",       s_feed [20],   *c_feed   =" R9=";  
   char                            s_ffeed[20],   *c_ffeed  =" F";
   char *dwell     = "DWELL",      s_dwell[20],   *c_dwell  =" R4=";
   char *nodrag    = "NODRAG",     s_nodrag[20],  *c_nodrag =" R13=";
   char *mmpr      = "MMPR"; 
   char *drill     = "DRILL";
   char *csink     = "CSINK";
   char *cskdia    = "CSKDIA";
   char *tlangl    = "TLANGL";
   char *holdia    = "HOLDIA";
   char *tap       = "TAP";
   char *clw       = "CLW";
   char *cclw      = "CCLW";
   char *bore      = "BORE";
   char *drag      = "DRAG";
   char *back      = "BACK";	 
   char *deep      = "DEEP";
   char *brkchp    = "BRKCHP";
   char *step      = "STEP";
   char *option    = "OPTION";
   char *m_goto    = "GOTO/";
   char *cycle_off = "CYCLE/OFF";
   char *cam       = "CAM",        s_cam[20],      *c_cam=" R7=";
   char                            s_dwesta[20],   *c_dwesta=" R0=" ;
   char 	 					   s_dirrot[20],   *c_dirrot = " R7="; /* default for l74 */
   char                            s_revrot[20],   *c_revrot = " R6="; /* default for l74 */
   char *cycle_on  = "CYCLE/",     s_step1[20], s_step2[20], *c_step1=" R1=", *c_step2=" R5=";
   char                            s_x[20],        *c_x=" R12=";
   char                            s_z[20],  	   *c_z=" R11=";
 
   double d_rapto=0.0;
   double d_fedto=0.0;
   double d_rtrcto=0.0;
   double d_feed=0.0; 
   double d_feed_old=0.0;
   double d_dwell=0.0;
   double d_cskdia=0.0;
   double d_tlangl=90.0;
   double d_nodrag=0.0; 
   double d_holdia=0.0;
   double d_step1=0.0;
   double d_step2=0.0;
   double d_cam=0.0;
   double d_dwesta=0.0;
   double d_dirrot=3.0;
   double d_revrot=4.0;
   double x_option=0.0;
   double y_option=0.0;
   double z_option=0.0;
   double x_option_old = -999999.99;
   double y_option_old = -999999.99;
   double z_option_old = -999999.99;
   double x, y, z, i_int, j_int, k_int;
   double c_old = -999999.99;
   double x_old_cycle = -999999.99; 
   double z_old_cycle = -999999.99;
   double Cr,C,Len;

   int i_rapto=1;
   int i_fedto=1;
   int i_rtrcto=1;
   int i_feed=1; 
   int i_dwell=1;
   int i_cskdia=1;
   int i_tlangl=1;
   int i_nodrag=1; 
   int i_holdia=1;
   int i_step1=1;
   int i_step2=1;
   int i_cam=1;
   int i_dirrot=1;
   int i_revrot=1;
   int i_dwesta=1;
   int i_option=0;
   int irc, ret_num, in;
   int i_drill=0;
   int i_csink=0;
   int i_deep=0;
   int i_brkchp=0;
   int i_tap=0;
   int i_bore=0;
   int i_drag=0;
   int i_no_drag=0;
   int i_l74=0;
   int i_l78=0;

   char menu_list_1[2][38]={"Using L74 cycle", "Using L78 cycle"};
   char *mess1="Select type thread cycle L74 or L78";
   char *mess2="Select type of thread";
   char menu_list_2[2][38]={"Right-hand thread", "Left-hand thread"};

/* define type of CYCLE */
   if(strstr(nc_block_in,drill) != NULL) i_drill = 1;
   if(strstr(nc_block_in,csink) != NULL) i_csink = 1;
   if(strstr(nc_block_in,deep)  != NULL) i_deep = 1;
   if(strstr(nc_block_in,brkchp)!= NULL) i_brkchp = 1;
   if(strstr(nc_block_in,tap)   != NULL) i_tap = 1;
   if(strstr(nc_block_in,bore)  != NULL) i_bore = 1;
   if(strstr(nc_block_in,drag)  != NULL) i_drag = 1;
   if(strstr(nc_block_in,nodrag)!= NULL) i_no_drag = 1;

   if(strstr(nc_block_in,option) != NULL) i_option = 1;
   else i_option = 0;
/* read parameters from CLS command */
   if((s1 = strstr(nc_block_in, rapto)) != NULL) 
   { 
      sscanf(s1+6, "%lf", &d_rapto); 
      i_rapto = 1; 
   } 
   if((s1 = strstr(nc_block_in, fedto)) != NULL) 
   { 
      sscanf(s1+6, "%lf", &d_fedto);	
      i_fedto=1; 
   }
   if((s1 = strstr(nc_block_in, rtrcto)) != NULL)  
   {
      if(strstr(nc_block_in, "AUTO") != NULL) 
      { 
         d_rtrcto = d_rapto; 
         i_rtrcto = 1;
      }
      else 
      { 
         sscanf(s1+7, "%lf", &d_rtrcto); 
         i_rtrcto = 1;	
      }
      
   }
   else if(i_rapto) 
   { 
      d_rtrcto = d_rapto;	
      i_rtrcto = 1;	
   }
   if((s1 = strstr(nc_block_in, mmpr)) != NULL)  
   { 
      sscanf(s1+5, "%lf", &d_feed ); 
      i_feed = 1; 
   }
   if((s1 = strstr(nc_block_in, mmpm)) != NULL)  
   { 
      sscanf(s1+5, "%lf", &d_feed );	
      i_feed = 1; 
   }
   if((s1 = strstr(nc_block_in, dwell)) != NULL)  
   { 
      sscanf(s1+6, "%lf", &d_dwell);	i_dwell=1;	
   }
   if((s1 = strstr(nc_block_in, step)) != NULL)  
   {
      sscanf(s1+5, "%lf", &d_step1); 
      i_step1 = 1;
      s2=strstr(s1+5,",");
      if(s2 != NULL && isdigit(*(s2+1))) 
      { 
         sscanf(s2+1, "%lf", &d_step2);	
         i_step2 = 1; 
      }
   }   
   if((s1 = strstr(nc_block_in, cskdia)) != NULL) 
   { 
      sscanf(s1+7, "%lf", &d_cskdia);	
      i_cskdia = 1; 
   }
   if((s1 = strstr(nc_block_in, tlangl)) != NULL) 
   { 
      sscanf(s1+7, "%lf", &d_tlangl);	
      i_tlangl = 1; 
   }
   if((s1 = strstr(nc_block_in, holdia)) != NULL) 
   { 
      sscanf(s1+7, "%lf", &d_holdia);	
      i_holdia = 1; 
   }
   if((s1 = strstr(nc_block_in, nodrag)) != NULL) 
   { 
      sscanf(s1+7, "%lf", &d_nodrag);	
      i_nodrag=1; 
   }
   if((s1 = strstr(nc_block_in, cam)) != NULL) 
   { 
       sscanf(s1+4, "%lf", &d_cam); 
       i_cam = 1;
   } 
   /* correction parameter by cycle type */
   if(i_csink)	
   {
      d_tlangl = d_tlangl/180*M_PI;
      d_fedto = (d_cskdia/2)/tan(d_tlangl/2);
   }
	if(i_tap)		/* CYCLE TAP L74 AND L78 */
	{
		irc = uc1603(mess1, 1, menu_list_1, 2);
		if(irc == 6)
		{ 
			i_l78 = 1;
			c_dirrot = (char *)malloc(sizeof(char)*5);
			strcpy(c_dirrot, " R6="); /* change key word for L78 cycle */
		}
		else i_l74 = 1;
		irc = uc1603(mess2, 1, menu_list_2, 2);
		if(irc == 6) 
		{ 
			d_dirrot = 4.0;	
			d_revrot = 3.0;	
		}
		else
		{ 
         	d_dirrot = 3.0; 
         	d_revrot = 4.0;	
		}
   }
   do			/* Loop until we read CICLE/OFF comman */
   {
      if((s1 = strstr(nc_block_in, m_goto)) != NULL)
      {
         ret_num = XYZ_input(s1+5, &x, &y, &z, &i_int, &j_int, &k_int, mill_mode);
         if(ret_num<6) {i_int=*i_old; j_int=*j_old; k_int=*k_old; }
		 else if(fabs(b_calculation(i_int,j_int,k_int) - b_old) > 0.01) 
         /* The changing of B coordinta is			
            is now allowed inside of cicle  */
         {PTP(" ERROR Changing of B coordinat inside cycle");}
         c_calculation(i_int,j_int,k_int,&Cr,&C);
         x_option = x, y_option = y, z_option = z;
         if (i_int == 1.0)
		 {  
	        if(y == 0.0 && z >= 0.0) { C = 0.0; Cr = 0.0; }
            else
            {
               Len = Len2(y,z); Cr = atan2(y/Len,z/Len); C = (Cr)*180/M_PI; 
               if (C<0.0) C=360.0 + C;
  			   z = Len2(z,y); y = 0.0;
            }
		 }
	     else C_rotate (&y,&z,Cr);  /* common rotate of C axis  why not B_axis ??? */
  	     C=c_correction(C); C=around (C); y=around(y); z=around(z); x=around(x);
	     if(C != c_old) sprintf(s_c," G68 C1=%.3lf", C);
         else strcpy (s_c,"");
         if(y != *y_old) sprintf(s_y," Y1=%.3lf", y);
         else strcpy(s_y,"");
         if(x != *x_old) sprintf(s_x," E1=%.3lf", z);     
         else strcpy(s_x,"");
         if(z != *z_old) sprintf(s_z," Z1=%.3lf", x + d_rapto);     
         else strcpy(s_z,"");
         if(type_of_goto != 0) 
         { 
            strcpy(COMMAND," G0"); 
            type_of_goto=0; 
         } 
         else strcpy(COMMAND,"");
         if((C != c_old || y != *y_old) && i_option == 0)
         {
            strcat(strcat(strcpy(nc_block_out,COMMAND),s_c),s_y);
		    PTP(nc_block_out);
         }
/*
         else if(i_option == 1)
         {
            if(y_option != y_option_old) sprintf(s_y," Y1=%.3lf", y_option);
            else strcpy(s_y,"");
            if(z_option != z_option_old) sprintf(s_x," E1=%.3lf", z_option);     
            else strcpy(s_x,"");
            if(x_option != x_option_old) sprintf(s_z," Z1=%.3lf", x_option + d_rapto);     
            else strcpy(s_z,"");
            strcat(strcat(strcat(strcpy(nc_block_out,COMMAND),s_x),s_y),s_z);
            PTP(nc_block_out);
         }
*/
         else if(i_option == 1)
         {
            if(y_option != y_option_old) 
            { 
                sprintf(s_y," Y1=%.3lf", y_option);
                strcat(strcpy(nc_block_out,COMMAND),s_y);
                PTP(nc_block_out);
            }
            else strcpy(s_y,"");
            if(z_option != z_option_old) sprintf(s_x," E1=%.3lf", z_option);     
            else strcpy(s_x,"");
            if(x_option != x_option_old) sprintf(s_z," Z1=%.3lf", x_option + d_rapto);     
            else strcpy(s_z,"");
         }
	  /* define modality of parameters */
	     if(i_dwesta)  
         { 
             sprintf(s_dwesta,"%s%.3lf",c_dwesta,d_dwesta); 
             i_dwesta=0; 
         } 
         else strcpy(s_dwesta,"");
         if(i_fedto)   
         { 
             sprintf(s_fedto, "%s%.3lf",c_fedto, fabs(d_fedto) ); 
             i_fedto =0; 
         } 
         else strcpy(s_fedto, "");
	     if(i_step1)   
         { 
             sprintf(s_step1, "%s%.3lf",c_step1, d_step1 ); 
             i_step1 =0; 
         } 
         else strcpy(s_step1, "");
	     if(i_rapto)   
         { 
             sprintf(s_rapto, "%s%.3lf",c_rapto, d_rapto ); 
             i_rapto =0; 
         } 
         else strcpy(s_rapto, "");
	     if(i_dwell)   
         { 
             sprintf(s_dwell, "%s%.3lf",c_dwell, d_dwell ); 
             i_dwell =0; 
         } 
         else strcpy(s_dwell, "");
	     if(i_step2)   
         { 
             sprintf(s_step2, "%s%.3lf",c_step2, d_step2 ); 
             i_step2 =0; 
         } 
         else strcpy(s_step2, "");
	     if(i_revrot)  
         { 
             sprintf(s_revrot,"%s%.3lf",c_revrot,d_revrot); 
             i_revrot=0; 
         } 
         else strcpy(s_revrot,"");
	     if(i_dirrot)  
         { 
             sprintf(s_dirrot,"%s%.3lf",c_dirrot,d_dirrot); 
             i_dirrot=0;
         } 
         else strcpy(s_dirrot,"");
	     if(i_cam)     
         { 
             sprintf(s_cam, "%s%.3lf", c_cam, d_cam); 
             i_cam = 0; 
         } 
         else strcpy(s_cam, "");
	     if(i_rtrcto)  
         { 
             sprintf(s_rtrcto,"%s%.3lf",c_rtrcto,d_rtrcto); 
             i_rtrcto = 0; 
         } 
         else strcpy(s_rtrcto,"");
	     if(i_feed)    
         { 
             sprintf(s_feed,  "%s%.3lf",c_feed,  d_feed  ); 
	         sprintf(s_ffeed, "%s%.3lf",c_ffeed, d_feed  ); 
             i_feed = 0; 
         } 
         else 
         { 
             strcpy(s_feed,  "");	
             strcpy(s_ffeed,  "");	
         }
         if(z != z_old_cycle && i_option == 0) 
         { 
             sprintf(s_z,"%s%.3lf", c_z, z);
         }
         else if(i_option == 1)
         {
             if(z_option != z_option_old)
             sprintf(s_z,"%s%.3lf", c_z, z_option);
             else strcpy(s_z, "");
         }
         else strcpy(s_z, "");
         if(x != x_old_cycle && i_option == 0) 
         { 
             sprintf(s_x,"%s%.3lf", c_x, x);             
         }
         else if(i_option == 1)
         {
             if(x_option != x_option_old)
             sprintf(s_x,"%s%.3lf", c_x, x_option);
             else strcpy(s_x, "");
         }
         else strcpy(s_x, "");
         if(i_nodrag)  
         { 
             sprintf(s_nodrag,"%s%.3lf",c_nodrag,d_nodrag); 
             i_nodrag=0; 
         } 
         else strcpy(s_nodrag,"");
         *x_old=x; *y_old=y; *z_old=z; c_old=C;	
         x_old_cycle=x; z_old_cycle=z;
         *i_old=i_int; *j_old=j_int; *k_old=k_int;
         x_option_old = x_option; 
         y_option_old = y_option;
         z_option_old = z_option;
/*=================== CICLE OUTPUT ===========================*/

/* L72 	*/
         if(i_drill && !i_deep && !i_brkchp && d_dwell > 0.0) 
         {
            strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcpy(  
		    nc_block_out,s_rapto),s_fedto),s_rtrcto),s_dwell),s_z),s_x)," L72"),s_ffeed);
         }
/* L71	*/
         if(i_drill && !i_deep && !i_brkchp && d_dwell == 0.0)
         {
            strcat(strcat(strcat(strcat(strcat(strcat(strcpy(  
		    nc_block_out,s_rapto),s_fedto),s_rtrcto),s_z),s_x)," L71"),s_ffeed);
         }
/* L70 	*/
         if(i_drill && i_brkchp  && !i_deep)
         {
            strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcpy(  
            nc_block_out,s_step1),s_rapto),s_fedto),s_rtrcto),s_dwell),s_step2),s_z),s_x)," L70"),s_ffeed);
         }
/* L73 	*/
         if(i_drill && !i_brkchp && i_deep)
         {
            strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcpy(  
		    nc_block_out,s_dwesta),s_step1),s_rapto),s_fedto),s_rtrcto),s_dwell),s_step2),s_z),s_x)," L73"),s_ffeed);
         }
/* L75	*/
         if(i_bore && !i_drag && !i_nodrag)
         {
            strcat(strcat(strcat(strcat(strcat(strcat(strcpy(  
            nc_block_out,s_rapto),s_fedto),s_rtrcto),s_z),s_x)," L75"),s_ffeed);
         }
/* L76	*/
         if(i_bore && (i_drag || i_nodrag))
         {
			PTP(" M3=3");
            strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcpy(  
		    nc_block_out,s_rapto),s_fedto),s_rtrcto),s_z),s_x),s_nodrag)," L76"),s_ffeed);
         }
/* L74 */
         if(i_tap && i_l74)
         {
            strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcpy(  
            nc_block_out,s_rapto),s_fedto),s_revrot),s_dirrot),s_rtrcto),s_feed),s_z),s_x)," L74");
         }
/* L78 */
         if(i_tap && i_l78)
         {
            strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcat(strcpy(  
            nc_block_out,s_rapto),s_fedto),s_dirrot),s_cam),s_rtrcto),s_feed),s_z),s_x)," L78");
         }
         PTP(nc_block_out);
      }
      else if((s1 = strstr(nc_block_in, cycle_off)) != NULL)
      {      
          i_option = 0;
          break;
      }
      else if((s1 = strstr(nc_block_in, cycle_on )) !=NULL )	/* new parameters of cycle */
      {
        /* read parameters from CYCLE/ON command */
         if((s1 = strstr(nc_block_in, option))!= NULL) i_option = 1;
         else i_option = 0;
         if((s1 = strstr(nc_block_in, rapto)) != NULL) { sscanf(s1+6, "%lf", &d_rapto); i_rapto=1; } 
         if((s1 = strstr(nc_block_in, fedto)) != NULL) { sscanf(s1+6, "%lf", &d_fedto);	i_fedto=1; }
         if((s1 = strstr(nc_block_in, rtrcto))!= NULL)  
 	     {
            if(strstr(nc_block_in, "AUTO") != NULL) { d_rtrcto = d_rapto; i_rtrcto = 1;}
            else { sscanf(s1+7, "%lf", &d_rtrcto); i_rtrcto = 1;	}
         }
         else if(i_rapto) { d_rtrcto = d_rapto;	i_rtrcto = 1;	}
         if((s1 = strstr(nc_block_in, mmpr)) != NULL)  { sscanf(s1+5, "%lf",    &d_feed ); i_feed =1; }
         if((s1 = strstr(nc_block_in, mmpm)) != NULL)  { sscanf(s1+5, "%lf",    &d_feed );	i_feed =1; }
         if((s1 = strstr(nc_block_in, dwell))!= NULL)  { sscanf(s1+6, "%lf",    &d_dwell);	i_dwell=1;	}
         if((s1 = strstr(nc_block_in, step)) != NULL)  
         {
            sscanf(s1+5, "%lf", &d_step1); i_step1=1;
            s2=strstr(s1+5,",");
            if(s2!=NULL && isdigit(*(s2+1)) ) { sscanf(s2+1, "%lf", &d_step2);	i_step2=1; }
          }   
          if((s1 = strstr(nc_block_in, cskdia)) != NULL) { sscanf(s1+7, "%lf", &d_cskdia);	i_cskdia=1; }
          if((s1 = strstr(nc_block_in, tlangl)) != NULL) { sscanf(s1+7, "%lf", &d_tlangl);	i_tlangl=1; }
          if((s1 = strstr(nc_block_in, holdia)) != NULL) { sscanf(s1+7, "%lf", &d_holdia);	i_holdia=1; }
          if((s1 = strstr(nc_block_in, nodrag)) != NULL) { sscanf(s1+7, "%lf", &d_nodrag);	i_nodrag=1; }
	      if((s1 = strstr(nc_block_in, cam   )) != NULL) { sscanf(s1+4, "%lf", &d_cam   ); i_cam   =1; } 
      /* correction parameter by cycle type */
         if(i_csink && (i_cskdia || i_tlangl)) 
         {	
             d_tlangl = d_tlangl/180*M_PI;
             d_fedto  = (d_cskdia/2)/tan(d_tlangl/2); 
             i_fedto  = 1; 
         }
      }
   }while ((in = uc4514(chanel_in, nc_block_in) >= 0));		 
   /* end of while cycle */
   i_option = 0; 
}

/* CIRCLE processing function */
/* Function converted CYRCLE APT command to C2/G3 or more G1 commands in
         milling mode*/
void circle_process_mill(double B,
                         double coord_x,  double coord_y,   double coord_z, 
                         double Center_x, double Center_y,  double Center_z, 
                         double Normal_x, double Normal_y,  double Normal_z, 
                         double End_x,    double End_y,     double End_z,
                         double radius,   double tolerance, double ratio, double angle)
{
   char COMMAND[4];
   char s_out[256];
   double Ex,Ey,Ez, Cx,Cy,Cz, Bx,By,Bz;
   double Lx,Ly,Lz, Mx,My,Mz, Nx,Ny,Nz;
   double Len;
   double FI,Rmin,Rmax,df;
   double xl,yl,zl,xt,yt,zt;
   double scalqr,rt;
   int steps,i,dir;	

   Bx = coord_x; By = coord_y; Bz = coord_z;
   Cx = Center_x; Cy = Center_y; Cz = Center_z;
   Ex = End_x; Ey = End_y; Ez = End_z;
   Nx = Normal_x; Ny = Normal_y; Nz = Normal_z;

	/* --> 1. Calculation of local system coordinat */
 
   Lx=Bx-Cx; 
   Ly=By-Cy; 
   Lz=Bz-Cz;
   Len=sqrt(Lx*Lx+Ly*Ly+Lz*Lz);
   Lx=Lx/Len; 
   Ly=Ly/Len; 
   Lz=Lz/Len;
   Mx=Ny*Lz-Nz*Ly; 
   My=Nz*Lx-Nx*Lz; 
   Mz=Nx*Ly-Ny*Lx;  /* M= N x L */

/* can we save the circle interpolation */

   strcpy(COMMAND,"");
   if(B==0.0 || B==90.0 || B==-90.0)
   {	
      if(fabs(fabs(Nz) - 1.00) < 0.00001)	/* The circle in XY plane */
      { 
         if(Nz > 0.0 && type_of_goto != 2)  
         { 
            strcpy(COMMAND," G2"); 
            type_of_goto=2; 
            dir= 1;
         }
         else if(Nz < 0.0 && type_of_goto !=3 )  
         { 
            strcpy(COMMAND," G3"); 
            type_of_goto=3;	
            dir=-1;
         }
         if(plane_of_circle != 1) 
         {   
            plane_of_circle  = 1; 
            PTP(" G16 Y1= Z1= E1= Z1="); 
         }
         sprintf(s_out, "%s Y1=%.3lf Z1=%.3lf J%.3lf K%.3lf",
                          COMMAND, Ey, Ex, Cy-By, Cx-Bx);
      }
      else if( fabs(fabs(Ny)-1.00) < 0.00001)	/* The circle in XZ plane */
      { 
   	     if(Ny > 0.0 && type_of_goto != 2)  
         { 
            strcpy(COMMAND," G2"); 
            type_of_goto=2; 
            dir= 1; 
         }
         else if(Ny < 0.0 && type_of_goto != 3)  
         { 
            strcpy(COMMAND," G3"); 
            type_of_goto=3;	
            dir=-1;
         }
		 if(plane_of_circle != 3) 
         {   
            plane_of_circle  = 3; 
            PTP(" G16 Z1= E1= E1= Z1="); 
         }
         sprintf(s_out, "%s E1=%.3lf Z1=%.3lf I%.3lf K%.3lf",
                         COMMAND, Ez, Ex, Cz-Bz, Cx-Bx);
      }
      else if(fabs(fabs(Nx) - 1.00) < 0.00001)	/* The circle in YZ plane */ 
      { 
         if(Nx > 0.0 && type_of_goto != 2)  
         { 
            strcpy(COMMAND," G2"); 
            type_of_goto=2; 
            dir= 1;
         }
         else if(Nx < 0.0 && type_of_goto != 3)  
         { 
            strcpy(COMMAND," G3"); 
            type_of_goto=3;	
            dir=-1;
         }
		 if (plane_of_circle != 2)	
         {   
            plane_of_circle  = 2; 
            PTP(" G16 E1= Y1= E1= Z1="); 
         }
         sprintf(s_out, "%s E1=%.3lf Y1=%.3lf I%.3lf J%.3lf",
                         COMMAND, Ez, Ey, Cz-Bz, Cy-By);
      }
      else PTP(" ERROR coordinat B=0 or 90 or -90 and circle are not one of the mains plane");
      PTP(add_cls(add_feedrate(s_out),nc_block_in));
   }
   else				/* Linear interpolation:
	           The method of Local System calculation gurunties 
               that the positive rotation from begining to end point! */
   {
      dir=1;
      PTP(" ERROR you have a linear intepolation of circle");
      PTP("       in case of uniting circle it will produce wrong NC programm");

      Bx=Bx-Cx; By=By-Cy; Bz=Bz-Cz; Ex=Ex-Cx; Ey=Ey-Cy; Ez=Ez-Cz;	/* Step calculation */
      
      FI=angle;
      Rmin=radius-tolerance*ratio;  Rmax=radius+tolerance*(1-ratio);
      df=2*acos(Rmin/Rmax); steps=(int)(FI/df+0.5)+1; /* +1 step for sure  */
      df=FI/steps; 

      zl=0.0;
      strcpy(COMMAND," G1");
      for(i = 1; i <= steps; i++)
      {
         if(i == steps)	
         { 
            xt=End_x; yt=End_y; zt=End_z;
         }
         else
         {		
            FI=i*df*dir;
            xl=Rmax*cos(FI); yl=Rmax*sin(FI); /* Local coordinat systems */ 
            xt=Lx*xl+Mx*yl+Nx*zl;	         /* Return to 3D coordiant system */
            yt=Ly*xl+My*yl+Ny*zl;
            zt=Lz*xl+Mz*yl+Nz*zl;	
            
            /* Check scalqr */
            Len=sqrt(xt*xt+yt*yt+zt*zt);
		    scalqr=((xt/Len)*Nx+(yt/Len)*Ny+(zt/Len)*Nz);															   
            if(fabs(scalqr) > 0.001) 
      	    {
      		   sprintf(s_out," ERROR point are not circle scalqr=%lf\n", scalqr);
			   PTP(s_out);
            }
            xt=xt+Cx; yt=yt+Cy; zt=zt+Cz;
            rt=sqrt((xt-Cx)*(xt-Cx)+(yt-Cy)*(yt-Cy)+(zt-Cz)*(zt-Cz));
       		if(fabs(rt-Rmax) > 0.001  && fabs(rt-radius)>0.001)  
            {
               sprintf(s_out," ERROR point are not circle radius=%lf rt=%lf\n", rt,radius);
			   PTP(s_out);
            }
         }
         sprintf (s_out,"%s E1=%.3lf Y1=%.3lf Z1=%.3lf",COMMAND, zt,yt,xt);
         PTP(add_cls(add_feedrate(s_out),nc_block_in));
         if (i==1) strcpy(COMMAND,"");
      }
   }
}

/* Service functions */
/* This function make an arounding of 3 digital after
   decimal point
   ===================================================  */
double around (double x)
{
   double y,arr;
   int    tol = 1000; /* mean three digital after decimal point */
   if(x > 0.0) arr = 0.5; else arr = -0.5;
   y = ((long int)(x*tol + arr))/(double)tol;
   if(y == (-0.0)) y = 0.0 ;
   return(y);
}

/* This function add CSL command to PTP output file
   ================================================  */
char *add_cls(char *so, char *si) /* produce debug input */
{
   int no,ni,i;
   int debug = 0;
   
   if(!debug) return(so);
   no = strlen(so); ni = strlen(si);
   if((no+ni) < 250)
   {
      if(no < 50)
	  {
	     for(i = no; i < 50; i++) so[i]=' '; so[50]='\0';
	  }
      strcat(strcat(strcat(so," ("),si),")");	
   }
   return (so);
}
/*- Function calculate C axis position -*/
void  c_calculation(double i, double j, double k, double *Cr_out, double *C_out)
{	
   double C,Cr;
	
   if(j == 0.0 && k == 0.0) Cr = 0.0 ;		
   else Cr = atan2(j,k);
   C = (Cr)*180.0/M_PI;
   if(C < 0) C = 360 + C;	
   else if(C > 360) C = C - 360;
   *Cr_out = Cr; *C_out = C;
}
/* Function calculate C angle */
void C_rotate(double *y, double *z, double c)	
{
   double y1,z1;
   
   y1 = (*y)*cos(c) - (*z)*sin(c);
   z1 = (*y)*sin(c) + (*z)*cos(c);		
   (*y) = y1; (*z) = z1;
} 
/*- Function calculate B axis position -*/
double b_calculation(double i, double j, double k)
{
   double angle_b;

   angle_b = -180.0/M_PI * asin(i);
   return (angle_b);
}
/* - Function calculate B angle -*/

void B_rotate (double *x, double *z, double xoff, double zoff, double b)
{
  double x1,z1;

  x1=  ((*x) - xoff)*cos(b) + ((*z)-zoff)*sin(b);
  z1= -((*x) - xoff)*sin(b) + ((*z)-zoff)*cos(b);
  *x = -x1; *z = z1;
}
/*- Function add block number to string -*/
char *add_number(char *s)
{	
   char s1[256];

   strcpy(s1, s);  
   sprintf(s, "N%04d%s", block_num, s1); 
   block_num++;
   if(block_num >= 9998) block_num = 1;
   return(s);
}

/* This function return od 2D vector length*/

double Len2(double x, double y) {return (sqrt(x*x+y*y));}	

/* This function return od 3D vector length*/

double Len3(double x, double y, double z) {return (sqrt(x*x+y*y*+z*z));}	

/* this function add number, delete all right zero and 
write the command to PTP file */
void PTP(char *s) 
{	
   char s1[256];
   sprintf(s1,"N%04d",block_num); 
   block_num++; 
   if(block_num >= 9998) block_num = 1;
   strcat(s1,s);
   delete_right_zero(s1);
   uc4524(chanel_out,s1);
}
/* This function add feedrate code to end of string*/
	
char *add_feedrate( char *s)
{
   if(fedrat_on) 
   {
      strcat(s,feedrate); 
      fedrat_on=0; 
   }
   return(s);
}
 
/* - This function remove rigth zero all nubmer in string -*/
void delete_right_zero(char *s) 
{
   int n, i, j, k, m, can_delete;
   char s1[256];

   n = strlen(s);
   for(i=0; i < 256; i++) s1[i] = '\0';
   for (i=0,j=0; i<n; )
   {
      if(s[i] != '.') 
      { 
         s1[j] = s[i]; 
         if(s1[j] == '\t') s1[j]=' '; j++; i++; 
      }
      else
      {
      for(k = i + 1; isdigit(s[k + 1]); k++);
         for(can_delete = 1, m = k; m >= i && can_delete; m--)
		 {
		    if(s[m]=='0' || s[m]=='.') s[m]='#';
            else can_delete=0;
         }
		 for(m = i; m <= k; m ++, i ++)
         {
         if(s[m] != '#') 
         { 
            s1[j] = s[i]; 
            j++; 
            }
	 }
      }
   }
   strcpy(s,s1);
}
/* This function ask a parameter after keyword and return
   1 in sucess or 0 in unsecess  */

int ask(char *s, char *key, double *x) 
{	
   char   *p;
   
   p = strstr(s,key);
   if(p != NULL) 
   { 
      sscanf(p+strlen(key),"%lf",x); 
      return(1); 
   }
   return (0);
}
/* This funcition try to unite 
several circle command to single command &*/

void circle_unite(double  Xc, double  Yc, double  Zc,
                  double  Nx, double  Ny, double  Nz, double R,
                  double *Xe, double *Ye, double *Ze, double *angle)
{

  int file_pos;	/* current position of CLS file */
  char s[256];  /* input buffer */

  double XC,YC,ZC, XE,YE,ZE, NX,NY,NZ, Rnew, new_angle;
  double tmp1,tmp2,tmp3;
  		
  file_pos=uc4534(chanel_in);
  uc4514(chanel_in, s); /* read a next GOTO command */
  if(strstr(s,"CIRCLE/") == NULL) uc4535(chanel_in,file_pos); 
  else
  { 
      sscanf(s+7,"%lf,%lf,%lf,%lf,%lf,%lf,%lf", 
             &tmp1,&tmp2,&tmp3,&tmp1,&tmp2,&tmp3,&Rnew);
      XYZ_input(s+7,&XC,&YC,&ZC,&NX,&NY,&NZ,mill_mode);
      if(XC == Xc && YC == Yc && ZC == Zc && 
         NX == Nx && NY == Ny && NZ == Nz && R == Rnew)
      {
         uc4514(chanel_in, s); /* read a next GOTO command */
         XYZ_input(s+5,&XE,&YE,&ZE,&tmp1,&tmp2,&tmp3,mill_mode);
	     new_angle=acos(((XE - Xc)*(*Xe - Xc) + (YE - Yc)*(*Ye - Yc) + (ZE - Zc)*(*Ze - Zc))/(R*R));
		 new_angle=*angle+new_angle;
         circle_unite(Xc,Yc,Zc, Nx,Ny,Nz, R, &XE, &YE, &ZE, &new_angle);
         *Xe=XE; *Ye=YE; *Ze=ZE; *angle=new_angle;
      }
      else uc4535(chanel_in, file_pos);
   }
}
/*- This function compare two coordinat systems -*/

int compare_msys(struct msys m1, struct msys m2)
{
  int i;

  for(i = 0; i < 9; i++) if(m1.p[i] != m2.p[i]) return (0);
  return (1);
}
/*- This function produce an ERROR message delete
   PTP file and stop processing  -*/

void PTP_STOP(char *message)				
{ 
   char command[200];  
  
   uc1601(message, 1);
   uc4540(chanel_out, 0);
   strcat (strcpy(command,"rm "),ptp_file);
   system(command);
}  
/* this function stop  processingan and close PTP file -*/
void PTP_CLOSE(char *message)				
{ 
  uc1601(message, 1);		
  uc4540(chanel_out, 0);
}  
/*- This function look next GOTO unitl END OF PATH
    end calculate the B coordinat in sucesses  
    return 1 in unsucesses  return 0 -*/									
int look_goto(double *b)       																										
{
   int file_pos,rezult;
   char   s[256];
   double x, y, z, i, j, k;
   file_pos = uc4534(chanel_in);
   do
   {
      uc4514(chanel_in, s);
      if(strstr(s, "END OF PATH") != NULL) return (0);
      if(strstr(s, "GOTO/") != NULL)
      {
         rezult = XYZ_input(s+5, &x, &y, &z, &i, &j, &k, mill_mode);
		 if(rezult==6)
		 {
		    *b = b_calculation(i,j,k);
            uc4535(chanel_in,file_pos);
            return (1);
         }
      }
   }while(1);
} 
/* this function reset modality of all coordinat */								
void reset_XYZC()  
{ 
  Xold = Yold = Zold = Cold = -9999999.999;
}

/* correction of C coordinat after using of ROTATE */
double c_correction(double c) 
{
   if(rotate == -999999.999) 
   {
      PTP(" ERROR: No ROTATE command in path with MSYS other then first path");
	  return(c);
   }
   return (c+rotate);
}

/*  This function read X,Y,Z coordinat from CLS file 
    return 3 if read X,Y,Z return 6 if read X,Y,Z,I,J,K
    return 0 in fail to read coordinat  */
int XYZ_input(char *p, double *x, double *y, double *z, 
              double *i, double *j, double *k, int mill_mode) 
{	
   int rezult;
   
   rezult = sscanf(p,"%lf,%lf,%lf,%lf,%lf,%lf",x,y,z,i,j,k); 
   if(mill_mode == -1) PTP(" ERROR: unknown mill_mode in XYZ_input function");
   if(mill_mode) 
   { 
       (*y)=-(*y); 
       (*j)=-(*j); 
   }
   return (rezult);
}

/*- This function read type of cutter compensation -*/
void f_compensation (char *s, int *comp, int *type_of_comp)
{
   char sout[256];
/*   char sout_1[256]; */
   *comp = f_cutcom(s);
   if(strstr(s, "OFF"  ) != NULL)                         *comp = 40;
   else if (strstr(s, "LEFT" ) != NULL)                 { *comp = 41;	*type_of_comp=41; } 
   else if (strstr(s, "RIGHT") != NULL)                 { *comp = 42;	*type_of_comp=42; }
   else if (strstr(s, "ON"   ) != NULL && *type_of_comp)  *comp = *type_of_comp;
   else PTP(" ERROR: unknown type of compensatation in CUTCOM/ON command");   
   /* NEXT STRINGS ADD FOR CORRECT VISUALIZATION IN UNISIM */
   /* START */
/*   
   if(mill_mode && Kold == 1)
   { 
      sprintf(sout_1, " G19");
      PTP(sout_1);
   }
   else if(mill_mode && Iold == 1)
   {
      sprintf(sout_1, " G17");
      PTP(sout_1);
   }
*/
   /* END */
   sprintf(sout, " G%d", *comp); PTP(sout);
}


#if defined(DEBUG)         /* This funcion is used for debugin without UNIGRAPHICS */
#include "ufun_emulation.c"
#endif

